;OBH2c - One Button Home 2
;September 6, 2012

;  This is identical to OBH2a EXCEPT for clock correction.
;     This clock correction is used ONLY when unit is disconnected from remote 
;     access.  That is, it is in the Display Clock Mode.  In this mode all relays
;     can still be toggled on or off at any given time by local control.

;  The clock correction is a counter which can be set to any interval.
;     When the count reaches overflow, a one second speed up or slow down is accomplished.     one second speed up of the clock at regular intervals.

;  This allows much better long term accuracy since time can be averaged over several
;     months with varying temperatures and other environmental changes.


;  The "a" version was to accomodate two-color LEDs which have negative voltage requirements to light
;  The "c" version was to accomodate accurate clock correction while off line as regular slave.

;OBH2 -- SLAVE with up to four relays ( Changes required in boot section for SLAVE operation! )

;Crystal frequency = 3,579,545 Hz

;NOTE:  This code is designed to cut both the MASTER and SLAVE units.
;         During bootup, it directs how chip will respond; thus,
;         you must select lines in bootup code to activate the chip
;         according to needs.  As a slave unit, you must also select
;         the active number of relays wired into the unit.   This chip
;         will accomodate one to four active slave unit relays.

;
;Modified May 15, 2010 for slave #7 - 4 relays
;Modified May 17, 2010 for ALL units - Lived In EEPROM location for time on/off
;Modified December 5, 2011 for new clock correction addition and alarm during display of time
;Modified September 6, 2012. R17, flag register pushed, changed, and popped without data being saved.
	;Used unused register r22 instead of r17.

;REVISED:  November 24, 2009 - Rework of relay 1 automatic off circuit (Command #25 modified to make entry
;  of minutes using dit (1 min), dah (10 min), and hold (1 hour) down counts

;Copyright 2011 by:

;Wayne Simister
;Automated Sound
;584 Cortez Street
;Salt Lake City, UT 84103
;wsimister@hotmail.com


;Following FUSES are set on ATMega8L chip:

;  Preserve EEPROM memory through the chip erase cycle
;  Brown-out detection level at VCC = 2.7V
;  Brown-out detection enabled
;  Ext. Crystal/Resonator High Freq; Start-up time 16K CK +64ms


.include "m8def.inc"


;R0 = 4 a.m. receive gate counter in Slave & Master
;R8 = 4 a.m. pulse in Slave gate counter
;R9 = 4 a.m. Slave and Master DOW pulse counter

;R10 = Random number counter used in slave section loop
;        incremented in slave loop and Button Push

;R14 = shared in both Master and Slave for Dusk/Dawn 

;R16, R22 = scratch registers

;Global Flag R17

;0 (1) Relay 1 status 1=ON
;1 (2) Relay 2 status 1=ON
;2 (4) Relay 3 status 1=ON
;3 (8) Relay 4 status 1=ON
;4 (16) Clock Correction Speed Up Flag
;5 (32) Pre Beep flag
;6 (64) Lived In flag 0=off
;7 (128) Night Light flag 0=off


;  R18 Pulse Detect Only flag

;0 (1) Send out 10usec pulse = 1
;1 (2) Pulse detected = 1
;2 (4) QuickFetch bit
;3 (8) Lived in 2 enable = 1
;4 (16) Lived in 2 in process flag
;5 (32) Receive dusk flag
;6 (64) Receive dawn flag
;7 (128) Burglar Alarm flag 1 = enabled


;  R19 Global Flag

;0 (1) Variable clock double flash flag
;1 (2) Beep flag
;2 (4) Clock Correction - Subtract ONE second flag
;3 (8)  Master (1) / Slave (0) flag
;4 (16) Killl delay flag
;5 (32) Clock Correction - Add ONE second flag
;6 (64)  One minute flag
;7 (128) One second flag

;  R21 Global Flag
 
;0 (1) Quick Alarm Once Only Flag
;1 (2) 
;2 (4)
;3 (8)
;4 (16)
;5 (32)
;6 (64)
;7 (128)


;   R26 Global Flag

;0 (1) 
;1 (2) 
;2 (4)
;3 (8)
;4 (16) One Minute Beep Flag
;5 (32) Lived In 3 enable 
;6 (64) Lived In 3 active
;7 (128) 


;   R29 Global Flag 

;0 (1) 4a.m. 1 = detection PIND,4
;1 (2) Proximity Detector 1 = dection PIND,5
;2 (4) Dusk/Dawn 1 = night, PIND,6
;3 (8) Burglar Alarm 1= detected  PIND,3
;4 (16) Dusk/Dawn SEND flag
;5 (32) 
;6 (64) Proximity Detector enable = 1
;7 (128) 4 a.m. quick inputs 1 second bypass

;SRAM for clock correction data $110-$118
;EEPROM for clock correction $10 -$14


;r30,r31 Automatic Off of Relay 1 count in seconds

;*****Initialization


rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1					
rjmp	START1				;compare input capture register ICR			
rjmp	TimeInterrupt		;compare register A -- One Second Crystal Clock Routine				
rjmp	START1				;compare register B
rjmp	START1				;timer 1 overflow
rjmp	START1

	
START1:			;set up stack pointer and initialize cold start

ldi 	r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16				; Initialize stack pointer to end of internal RAM

ldi		r16,high(RAMEND)	; Load high byte address of end of RAM
out		SPH,r16				; Initialize stack pointer to end of RAM

ser		r16			;Set to all ones for out port direction as output
out		DDRB,r16	;Set PORTB to output (only 0-5 used)
out		DDRC,r16	;Set PORTC to output (only 0-5 used)

clr		r16
out		PortB,r16	;LEDs are off when port pins are low

sbi		PortB,1		;off RED LED
sbi		PortB,2		;off GREEN LED	;These two steps needed for "a" version (LEDs need negative ON voltage)

out		PortC,r16	;Off all port C out lines

;---------------
rcall	Delay2sec	;allow capacitors to charge and ready system
rcall	Delay2sec
;---------------

in		r16,TIMSK	;enable 16-bit match on overflow
ori		r16,16		;set bit 4(16) of TIMSK (timer 1 compare register A)         		
out		TIMSK,r16

in		r16,TCCR1B
ori		r16,11		; divide by 64(bits 0,1) high for prescaler, zero upon match 
					;  zero counter upon match of A (bit 3(8))
out		TCCR1B,r16                                      

ldi		r16,218		;high byte = 218 
out		OCR1AH,r16

ldi		r16,123		;default selected at 123 for this particular crystal
out		OCR1AL,r16	;loads the compare register A for timer 1 (1 second)
					; -- clock adj. to run slightly slow (for Corr Purp)


;-----------------------

clr		r19


;**** Following code to be changed for MASTER or SLAVE operation
;**** Use line "sbr  r19,8 for MASTER OPERATION
;**** DO NOT activate this line for SLAVE OPERATION

;sbr		r19,8				;set r19,3(8) for MASTER operation



sbrs	r19,3				;skip if r19,3(8) is set for Master
rjmp	START1slave


START1master:

sei					;ON interrupts for crystal clock timing

ldi		R16,192		;Set bits 0, 1, 2, 3, 4, 5 as inputs, 6,7 as out (MASTER)


out		DDRD,R16	;execute data direction with bits 0, 1, 2, 3, 4, 5 as inputs
out		PORTD,R16	;execute to set out ports high, and in ports low with
					; no input resistor

ldi		r16,1		;master unit has only ONE AC switching relay
sts		$006F,r16			;store active relay number in SRAM

rjmp	START1continue


START1slave:

cli					;OFF interrupts - use timing from control/data line

; ****Select ONE of the following lines to activate one to four relays in SLAVE

;ldi	r16,1			;Use this line for ONE relay

;ldi	r16,2		;Use this line for TWO relays  
;ldi	r16,3		;Use this line for THREE relays 
ldi		r16,4		;Use this line for FOUR relays 

sts		$006F,r16		;put number of active relays into SRAM

ldi		r16,250		;Set bits 0, 2 as inputs rest as out (SLAVE)

out		DDRD,R16	;execute data direction with bits 0, 2 as inputs, rest as outputs
out		PORTD,R16	;execute to set out ports high, and in ports low with
					; no input resistor


;--------------

START1continue:

rcall	SomeDefaultSettings


;Following initializes clock time at 8:00 a.m. at RESET

ldi		r16,1		;High Time Byte
sts		$0069,r16
ldi		r16,224		;Low Time Byte
sts		$0068,r16
ldi		r16,1		;set Day of Week to Sunday (1)
sts		$006A,r16
clr		r16
sts		$006B,r16	;zero seconds counter

sts		$0119,r16
sts		$011A,r16
sts		$011B,r16	;zero Display Clock alarm and 60 minute ON counter


rcall	ReadInUnitNumbers					;Fetch assigned unit numbers to relays from EEPROM
rcall	FetchLivedInOffTime					;Fetch Lived In Off time from EEPROM
rcall	ReadInClockCorrectionInterval		;Fetch interval for one second speed up
rcall	GrabAutoOffTimeInEEPROM
rcall	ClearRelayData
rcall	ClearAllAlarms

rcall	OffAllDuskDawnFunctions


;------------------
MainStartUp:
;------------------


sbrs	r19,3		;skip if r19,3(8) is set (Master)
rjmp	SlaveLoopA	;it is a SLAVE unit


;--------------------------------------------------------
;--------------------------------------------------------

MasterLoopA:

sei							;ON interrupts (Master operation)

rcall	OneSecondFlagStall	;sync up for MASTER clean start


MasterLoop:

sbis	PIND,2				;skip if PIND,2(4) is set (no touch of plate)
rjmp	DecodeButtonPush

rcall	QuickInputsCheck


MLB2:

sbrs	r19,7		;skip if r19,7(128) is set (one second flag)
rjmp	MasterLoop

rjmp	Loop0		;jump to Loop0


;---------------------------------

SlaveLoopA:

cli					;OFF interrupts
					;  Slave gets timing from power/data line

rcall	SyncUp		;sync up to incoming pulses for clean start


SlaveLoop:

sbis	PIND,2				;skip if PIND,2 (touch plate not touched)
rjmp	DecodeButtonPush

sbic	PIND,0				;skip if PIND,0 clear (pulse received)
rjmp	SlaveLoop
rjmp	Loop0				;jump to Loop0, 5 slot unit check loop


;--------------------

; 10 HOLD DOWN counts rjmps into this section


ProgramCommandStall:	;stalls with green LED flashing to await Program
						; command entry

push	r16
cbi		PORTB,2			;on green LED


PCStallStart:

ldi		r16,20			;toggle every 1/3 second


PCStalla:

rcall	Delay80th
sbis	PIND,2			;skip if touch plate is not touched
rjmp	PCStallc


PCStallb:

dec		r16
brne	PCStalla


PCStallToggle:

sbic	PortB,2	
rjmp	PCStallToggle1

sbi		PortB,2			;off green LED

rjmp	PCStallToggleEnd


PCStallToggle1:

cbi		PortB,2			;on green LED


PCStallToggleEnd:

rjmp	PCStallStart


PCStallc:

sbi		PortB,2			;off green LED
pop		r16


rjmp	ProgramCommand


;-------------------------------

ProgramCommand:		;Uses dit-dah low byte count -- to access these commands

rcall	ButtonPush	;get command entry (Interrupts have been turned on)

lds		r16,$00EA		;get total count, LS byte


PCOM1:			

cpi		r16,1					;1 entry
brne	PCOM2


rjmp	PCOMEnd


;------------------------


PCOM2:		

cpi		r16,2					;2 entry 
brne	PCOM3



rjmp	PCOMEnd


;------------------------

PCOM3:			

cpi		r16,3			;3 entry
brne	PCOM4



rjmp	PCOMEnd


;------------------------

PCOM4:

cpi		r16,4				;4 entry
brne	PCOM5



rjmp	PCOMEnd


;------------------------

PCOM5:			;OFF all dusk/dawn operations - cancels operation

cpi		r16,5			;5 entry
brne	PCOM6

rcall	OffAllDuskDawnFunctions
rcall	ShortBeep2

rjmp	PCOMEnd


;------------------------

PCOM6:			;Toggle lived in mode on/off (on at dusk, off at programmed time)

cpi		r16,6			;6 entry
brne	PCOM7

rcall	LivedInToggle	;Toggle Live In (on dusk, off at preset off time)

rjmp	PCOMEnd


;------------------------

PCOM7:			;Toggle lived in 2 on/off.  Cycles light on and off during
				;  evening hours to approximately programmed time off

cpi		r16,7			;7 entry
brne	PCOM8

rcall	LivedInTwoToggle

rjmp	PCOMEnd


;------------------------

PCOM8:			;Toggle night light mode on/off

cpi		r16,8			;8 entry
brne	PCOM9

rcall	NiteLiteToggle	;rcall to routine for work


rjmp	PCOMEnd


;------------------------

PCOM9:			

cpi		r16,9			;9 entry
brne	PCOM10


rjmp	PCOMEnd


;------------------------

PCOM10:			;Erase old and program new alarm times into SRAM

cpi		r16,10				;10 entry
brne	PCOM11

			;Program new series of alarm times into SRAM (old cleared)

;FIVE entries required 
			; 1) 1= EXIT, 2=on command, 3=OFF command. 4=OFF toggle
			; 2) 1= every day, 2 - 8 = Sunday (2) through Saturday (8)
			; 3) Unit number - Actual Unit No.  No need to add one to entry here
			; 4) (24 hour format) 1=0 hour, 2=1 hour, 3=2 hours, etc.
			; 5) Minutes 1=0 minutes, 2=1 minute, 3=2 minutes, etc.
			
rcall	EnterAlarmTimeData	;rcall to routine for work

rjmp	PCOMEnd


;------------------------

PCOM11:			;Add to alarm times already in SRAM

cpi		r16,11			;11 entry
brne	PCOM12

			;Add to series of alarm times already stored in SRAM
				; book mark is not cleared
			; 1) 1= EXIT, 2=on command, 3=OFF command. 4=OFF toggle
			; 2) 1= every day, 2 - 8 = Sunday (2) through Saturday (8)
			; 3) Unit number - 1 = 0,  2 = 1,  3 = 2,  etc.
			; 4) (24 hour format) 1=0 hour, 2=1 hour, 3=2 hours, etc.
			; 5) Minutes 1=0 minutes, 2=1 minute, 3=2 minutes, etc.

rcall	EnterAlarmTimeData2	;rcall to routine for work

rjmp	PCOMEnd


;------------------------

PCOM12:			;Clear all alarm times in SRAM

cpi		r16,12			;12 entry
brne	PCOM13

rcall	ClearAllAlarms	;rcall to routine for work

rjmp	PCOMEnd


;------------------------

PCOM13:			;Puts alarm times in SRAM into EEPROM for permanent storage.

cpi		r16,13			;13 entry
brne	PCOM14

rcall	SRAMtoEEPROM	;rcall to routine for work

rjmp	PCOMEnd


;------------------------

PCOM14:			;Puts EEPROM stored alarm times into SRAM

cpi		r16,14			;14 entry
brne	PCOM15

rcall	EEPROMtoSRAM	;rcall to routine for work

rjmp	PCOMEnd


;------------------------

PCOM15:			;Clear all alarm times in EEPROM

cpi		r16,15			;15 entry
brne	PCOM16

rcall	ClearEEPROMalarms

rjmp	PCOMEnd


;------------------------

PCOM16:			

cpi		r16,16			;16 entry
brne	PCOM17

rjmp	PCOMEnd


;------------------------

PCOM17:			

cpi		r16,17			;17 entry
brne	PCOM18

rjmp	PCOMEnd


;------------------------

PCOM18:			

cpi		r16,18			;18 entry
brne	PCOM19

rjmp	PCOMEnd


;------------------------

PCOM19:			

cpi		r16,19			;19 entry
brne	PCOM20

rjmp	PCOMEnd


;------------------------

PCOM20:			;toggles burlar alarm mode on/off (offs other modes) 

cpi		r16,20			;20 entry
brne	PCOM21

rcall	BurglarAlarmToggle 	;rcall to routine for work


rjmp	PCOMEnd


;------------------------

PCOM21:			;Toggles proximity detector mode on or off.  All other modes shut off

cpi		r16,21			;21 entry
brne	PCOM22

rcall	ProximityToggle	;rcall to routine for work

rjmp	PCOMEnd


;------------------------

PCOM22:			

cpi		r16,22			;22 entry
brne	PCOM23

rjmp	PCOMEnd


;------------------------

PCOM23:	

cpi		r16,23			;23 entry
brne	PCOM24

rjmp	PCOMEnd


;------------------------

PCOM24:								;

cpi		r16,24			;24 entry
brne	PCOM25

rjmp	PCOMEnd


;------------------------

PCOM25:			;Disable/enable this unit turning off automatically
				;1=turn off delay, 2 minutes to 18.2 hours.


cpi		r16,25			;
brne	PCOM26

rcall	AutoOffRelay1Control	;1 = disable, 2 or more = time in minutes (Minimum of two minutes time)

rjmp	PCOMEnd


;------------------------

PCOM26:					;Set clock time off for Lived In routine	

cpi		r16,26			;26 entry
brne	PCOM27

rcall	StoreLivedInTimeOff		;Set lived in time off for relay 1

rjmp	PCOMEnd


;------------------------

PCOM27:					;Set clock in 24 hour format - THREE entries needed
						;  DOW: 2=Sun, 3=Mon, 4=Tue, 5=Wed, 6=Thr., 7=Fri., 8= Sat
						;  1=0 hrs., 2=1 hr., 3=2 hrs., etc (offset of one)
						;  1=0 mins., 2=1 min., 3= 2 mins., etc.  (offset of one)

cpi		r16,27			;27 entry
brne	PCOM28

rcall	SetClock		

rjmp	PCOMEnd


;------------------------

PCOM28:			;Display Clock
				;This routine is released by longer touch of touch plate
				;Short touch of plate during this routine toggles relay 1 on/off
				;No external commands can be received while in this mode.


cpi		r16,28			;28 entry --- DISPLAY CLOCK section
brne	PCOM29

rcall	DisplayClock		;rcall DisplayClock routine through ClockSelect

rjmp	PCOMEnd


;------------------------

PCOM29:

cpi		r16,29			;29 entry
brne	PCOM30

rjmp	PCOMEnd


;------------------------

PCOM30:

cpi		r16,30			;30 entry
brne	PCOM31

rjmp	PCOMEnd


;------------------------

PCOM31:

cpi		r16,31
brne	PCOM32

rcall	SetUnitNumberRly1		;rcall to routine for work

rjmp	PCOMEnd


;------------------------

PCOM32:

cpi		r16,32
brne	PCOM33

rcall	SetUnitNumberRly2	;rcall to routine for work

rjmp	PCOMEnd


;---------------------------

PCOM33:

cpi		r16,33				
brne	PCOM34

rcall	SetUnitNumberRly3	;rcall to routine for work

rjmp	PCOMEnd


;------------------------------

PCOM34:

cpi		r16,34
brne	PCOM35

rcall	SetUnitNumberRly4

rjmp	PCOMEnd


;-----------------------------

PCOM35:

cpi		r16,35
brne	PCOM36

rcall	SetBATimeOn		;Set time on for relay 1 when burglar alarm triggered

rjmp	PCOMEnd


;------------------------------

PCOM36:

cpi		r16,36
brne	PCOM37

rcall	SetProxTimeOn	;Set time on for relay 1 when proximity is triggered

rjmp	PCOMEnd


;-----------------------------

PCOM37:				;set clock countdown divider low byte

cpi		r16,37
brne	PCOM41

rcall	SetLivedIn2OnOffProportions	;1 = 10 min on, 74 min off; 15 = 74 min on, 10 min off

rjmp	PCOMEnd


;----------------------------

PCOM41:			;set speed up or slow down of clock every XX seconds
				;One second is either subtracted or added at end of inte

;  Five entries required:  First is 16,777,216 units.  Second is 65536 units.thousands.
;	  Third is 256 units,  Fourth is single units.  Last is 0 = no change
;	  1 = slow clock by one second.   2 = speed up clock by one second.

cpi		r16,41
brne	PCOM43

rcall	SetClockOneSecondCorrection	;rcall to routine for work

rjmp	PCOMEnd


;-------------------------------

PCOM43:					;Allows setting of dusk/dawn potentiometer

cpi		r16,43
brne	PCOM44

rcall	TestDuskDawn	;test setting of potentiometer for dusk/dawn detection
						; green=day,  red=night

rjmp	PCOMEnd


;-----------------------------------

PCOM44:

cpi		r16,44
brne	PCOM50

rcall   LoadDefaultUnitNumbers	;Load unit number 31, 32, 33, 34 into EEPROM 

rjmp	PCOMEnd


;------------------------------

PCOM50:

cpi		r16,50
brne	PCOMEnd

rcall	DoDefaultReset	;Assign Unit No. 100 as default, Clear all flags, Set no auto off for relay 1, Clear all SRAM alarm
						;  times,  
						;  Clear all clock correction values in EEPROM and SRAM.
						
rjmp	PCOMEnd


;-------------------------------


PCOMEnd:

ldi		r16,2			;prepare wag routine for two wags of red green LED
rcall	Wag

sbrs	r19,3			;skip if r19,3(8) is set (master)
rjmp	SlaveLoopA		;jump after sync up into SlaveLoop
rjmp	MasterLoopA		;jump after sync up into MasterLoop


;------------------------------

;LOOPS . . . . . . 


Loop0:	;executed only once at beginning of loop series

clr		r16	
sts		$0071,r16	;clear receive command for unit 0 commands
sts		$0073,r16	;clear relay 1 receive command
sts		$0075,r16	;clear relay 2 receive command
sts		$0077,r16	;clear relay 3 receive command
sts		$0087,r16	;clear relay 4 receive command

cbr	r19,128	;clear one second flag

push	r20	;Unit number counter
push	r23	;
push	r24	;send byte
push	r25	;received byte


;After start pulse, a series of five bits are sent down the 
;control line.  First is the ON pulse bit, 2nd=Off Pulse, 3rd=
;toggle, 4th=return bit if relay off, 5th=spare (not used)

;555 clock cycles= 110 + 45 useconds
;394 clock cycles= 110 useconds
;358 clock cycles= 100 useconds
;322 clock cycles= 90 useconds
;161 clock cycles= 45 useconds

;A pre and post period of 161 clock cycles precede and end each
; 110 usecond "single pulse" unit.  

clr		r20		;clear unit number


Loop0branch1:

sbrs	r19,3	;skip if r19,3(8) is set (master)
rjmp	Loop0Branch2

rcall	Pulse60usec		;GENERATE START PULSE (229 clock cycles)
rcall	Delay100
rcall	Delay100
rcall	Delay98

rjmp	Loop1pre		;555 clock cycles to here


Loop0Branch2:			;slave

rcall	Delay100
rcall	Delay100
rcall	Delay100
rcall	Delay100
rcall	Delay100
rcall	Delay26

rjmp	Loop1pre		;555 clock cycles to here


;--------------------------------------

Loop1pre:

cbr		r18,1 		;clear r18,0(1) send bit

sbis	PIND,2		;skip if TP not touched 
rjmp	LoopExit	;jump out for touch plate decoding

sbrs	r19,3		;skip if r19,3(8) (master)
rjmp	L1P1c		;slave
rjmp	L1P1b		;master


L1p1b:

rcall	QuickInputsCheck	;23 clock cycles
rjmp	L1P1d


L1P1c:

rcall	Delay24
rjmp	L1P1d


L1P1d:

cpi		r20,0
breq	L1P2
rjmp	L1P1


L1P2:

sbrs	r19,3	;skip if r19,3(8) is set Master
rjmp	L1P3


L1P4:			;master

lds		r24,$0083	;load zero command
sbrs	r24,0	;skip if r24,0(1) is set
rjmp	L1P5
rjmp	L1P6


L1P6:		;send pulse

sbr		r18,1		;set r18,0(1) for puslse send
cbr		r24,1		;clear $0083,0(1)
sts		$0083,r24	;store back away
rcall	Delay100
rcall	Delay12
rjmp	Loop1post


L1P3:

rcall	Delay100
rcall	Delay21
rjmp	Loop1post


L1P1:

lds		r16,$0080	;get unit number from touch plate entry
cp		r16,r20
breq	L1P7
rjmp	L1P8


L1P7:

lds		r24,$0081	;command put touch plate command in send
sbrs	r24,0	;skip if r24,0(1) is set
rjmp	L1P9	;out
rjmp	L1P10


L1P10:

sbr		r18,1	;set send bit, r18,0(1)
rcall	Delay100
rcall	Delay11
rjmp	Loop1post


L1P5:

rcall	Delay100
rcall	Delay17
rjmp	Loop1post


L1P8:

rcall	Delay100
rcall	Delay17
rjmp	Loop1post


L1P9:

rcall	Delay100
rcall	Delay13
rjmp	Loop1post


;---------------------------------------------------

Loop1post:

rcall	SinglePulse	;394 clock cycles

sbrs	r18,1	;skip if r18,1(2) set (pulse received bit)
rjmp	L1st4
rjmp	L1st1


L1st1:

cbr		r18,2	;clr r18,1(2) pulse rcvd bit
cpi		r20,0	
breq	L1st2a
rjmp	L1st3


L1st2a:

rjmp	L1st2


L1st2:

sbrs	r19,3	;skip if r19,3(8) is set (master)
rjmp	L1st2b	;slave


L1stc:

rcall	Delay100
rcall	Delay47
rjmp	Loop2pre


L1st2b:

lds		r16,$0071
ori		r16,1
sts		$0071,r16
rcall	Delay100
rcall	Delay41

rjmp	Loop2pre


L1st4:

rcall	Delay100
rcall	Delay56

rjmp	Loop2pre


L1st3:

lds		r16,$0072
cp		r16,r20
breq	L1st5a
rjmp	L1st6


L1st5a:

rjmp	L1st5


L1st6:

lds		r16,$0074
cp		r16,r20
breq	L1st7a
rjmp	L1st8


L1st7a:

rjmp	L1st7


L1st8:

lds		r16,$0076
cp		r16,r20
breq	L1st9a
rjmp	L1st10


L1st9a:

rjmp	L1st9


L1st10:

lds		r16,$0086
cp		r16,r20
breq	L1st11a
rjmp	L1st12


L1st11a:

rjmp	L1st11


L1st12:

rcall	Delay100	
rcall	Delay26
rjmp	Loop2pre


L1st5:		;relay 1

lds		R16,$0073		;relay 1 ON
ori		r16,1		;ON bit
sts		$0073,r16

rcall	Delay100
rcall	Delay38
rjmp	Loop2pre


L1st7:		;relay 2

lds		r16,$0075
ori		r16,1		;Relay 2 on
sts		$0075,r16
rcall	Delay100
rcall	Delay32
rjmp	Loop2pre


L1st9:		;relay 3

lds		r16,$0077
ori		r16,1		;Relay 3 ON
sts		$0077,r16
rcall	Delay100
rcall	Delay26
rjmp	Loop2pre


L1st11:		;relay 4

lds		r16,$0087
ori		r16,1		;set relay 4 ON bit
sts		$0087,r16
rcall	Delay100
rcall	Delay20
rjmp	Loop2pre



;--------------------------------------------


Loop2pre:

cbr		r18,1 		;clear r18,0(1) send bit

sbis	PIND,2		;skip if TP not touched 
rjmp	LoopExit	;jump out for touch plate decoding

sbrs	r19,3		;skip if r19,3(8) (master)
rjmp	L2P1c		;slave
rjmp	L2P1b		;master


L2p1b:

rcall	QuickInputsCheck
rjmp	L2P1d


L2P1c:

rcall	Delay24
rjmp	L2P1d


L2P1d:

cpi		r20,0
breq	L2P2
rjmp	L2P1


L2P2:

sbrs	r19,3	;skip if r19,3(8) is set Master
rjmp	L2P3


L2P4:			;master

lds		r24,$0083	;load zero command
sbrs	r24,1	;skip if r24,1(2) is set
rjmp	L2P5
rjmp	L2P6


L2P6:		;send pulse

sbr		r18,1		;set r18,0(1) for puslse send
cbr		r24,2		;clear $0083,1(2)
sts		$0083,r24	;store back away
rcall	Delay100
rcall	Delay12
rjmp	Loop2post


L2P3:

rcall	Delay100
rcall	Delay21
rjmp	Loop2post


L2P1:

lds		r16,$0080	;get unit number from touch plate entry
cp		r16,r20
breq	L2P7
rjmp	L2P8


L2P7:

lds		r24,$0081	;command put touch plate command in send
sbrs	r24,1	;skip if r24,1(2) is set
rjmp	L2P9	;out
rjmp	L2P10


L2P10:

sbr		r18,1	;set send bit, r18,0(1)
rcall	Delay100
rcall	Delay11
rjmp	Loop2post


L2P5:

rcall	Delay100
rcall	Delay17
rjmp	Loop2post


L2P8:

rcall	Delay100
rcall	Delay17
rjmp	Loop2post


L2P9:

rcall	Delay100
rcall	Delay13
rjmp	Loop2post


;---------------------------------------------------

Loop2post:

rcall	SinglePulse	;394 clock cycles

sbrs	r18,1	;skip if r18,1(2) set (pulse received)
rjmp	L2st4
rjmp	L2st1


L2st1:

cbr		r18,2	;clr r18,1(2) pulse rcvd bit
cpi		r20,0	
breq	L2st2a
rjmp	L2st3


L2st2a:

rjmp	L2st2


L2st2:

sbrs	r19,3	;skip if r19,3(8) is set (master)
rjmp	L2st2b	;slave


L2stc:

rcall	Delay100
rcall	Delay47
rjmp	Loop3pre


L2st2b:

lds		r16,$0071
ori		r16,2		;Set bit 1(2)
sts		$0071,r16
rcall	Delay100
rcall	Delay41

rjmp	Loop3pre


L2st4:

rcall	Delay100
rcall	Delay56
rjmp	Loop3pre


L2st3:

lds		r16,$0072
cp		r16,r20
breq	L2st5a
rjmp	L2st6


L2st5a:

rjmp	L2st5


L2st6:

lds		r16,$0074
cp		r16,r20
breq	L2st7a
rjmp	L2st8


L2st7a:

rjmp	L2st7


L2st8:

lds		r16,$0076
cp		r16,r20
breq	L2st9a
rjmp	L2st10


L2st9a:

rjmp	L2st9


L2st10:

lds		r16,$0086
cp		r16,r20
breq	L2st11a
rjmp	L2st12


L2st11a:

rjmp	L2st11


L2st12:

rcall	Delay100	
rcall	Delay26
rjmp	Loop3pre


L2st5:		;relay 1

lds		R16,$0073		;relay 1 OFF
ori		r16,2		;OFF bit
sts		$0073,r16

rcall	Delay100
rcall	Delay38
rjmp	Loop3pre


L2st7:		;relay 2

lds		r16,$0075
ori		r16,2		;Relay 2 off
sts		$0075,r16
rcall	Delay100
rcall	Delay32
rjmp	Loop3pre


L2st9:		;relay 3

lds		r16,$0077
ori		r16,2		;Relay 3 OFF
sts		$0077,r16
rcall	Delay100
rcall	Delay26
rjmp	Loop3pre


L2st11:		;relay 4

lds		r16,$0087
ori		r16,2		;set relay 4 OFF bit
sts		$0087,r16
rcall	Delay100
rcall	Delay20
rjmp	Loop3pre


;---------------------------------------------------------------


Loop3pre:

cbr	 	r18,1 		;clear r18,0(1) send bit

sbis   	PIND,2		;skip if TP not touched 
rjmp   	LoopExit	;jump out for touch plate decoding

sbrs   	r19,3		;skip if r19,3(8) (master)
rjmp   	L3P1c		;slave
rjmp   	L3P1b		;master


L3p1b: 

rcall	QuickInputsCheck  ;(23 clock cycles)
rjmp   	L3P1d


L3P1c: 

rcall	Delay24
rjmp	L3P1d


L3P1d:  

cpi		r20,0
breq	L3P2
rjmp	L3P1


L3P2:   

sbrs   	r19,3	;skip if r19,3(8) is set Master
rjmp   	L3P3


L3P4:		;master

lds	  	r24,$0083	;load zero command
sbrs  	r24,2	;skip if r24,2(4) is set
rjmp  	L3P5
rjmp  	L3P6


L3P6:	;send pulse

sbr	  	r18,1		;set r18,0(1) for pulse send
cbr		r24,4		;clear $0083,2(4)
sts		$0083,r24	;store back away
rcall	Delay100
rcall	Delay12
rjmp	Loop3post


L3P3:  			;slave zero command

rcall	Delay100
rcall	Delay21
rjmp	Loop3post   


L3P1:   

lds   	r16,$0080	;get unit number from touch plate entry
cp	 	r16,r20
breq 	L3P7
rjmp 	L3P8


L3P7:

lds	 	r24,$0081	;command put touch plate command 
sbrs  	r24,2	;skip if r24,2(4) is set
rjmp  	L3P9	;out
rjmp  	L3P10


L3P10:  

sbr		r17,32	;set r17,5(32), beep flag (later reset if
				;  no pulse received in L4ps)
sbr  	r18,1	;set send bit, r18,0(1)
rcall  	Delay100
rcall	Delay10
rjmp	Loop3post


L3P5:

rcall	Delay100
rcall	Delay17
rjmp	Loop3post


L3P8:

rcall	Delay100
rcall	Delay17
rjmp	Loop3post


L3P9:

rcall	Delay100
rcall	Delay13
rjmp	Loop3post


;---------------------------------------------------

Loop3post:

rcall	SinglePulse	;394 clock cycles

sbrs	r18,1	;skip if r18,1(2) set (pulse received)
rjmp	L3st4
rjmp	L3st1


L3st1:

cbr		r18,2	;clr r18,1(2) pulse rcvd bit
cpi		r20,0	
breq	L3st2a
rjmp	L3st3


L3st2a:

rjmp	L3st2


L3st2:

sbrs	r19,3	;skip if r19,3(8) is set (master)
rjmp	L3st2b	;slave


L3stc:

rcall	Delay100
rcall	Delay47
rjmp	Loop4pre


L3st2b:

lds		r16,$0071
ori		r16,4		;set bit 2(4)
sts		$0071,r16
rcall	Delay100
rcall	Delay41
rjmp	Loop4pre


L3st4:

rcall	Delay100
rcall	Delay56
rjmp	Loop4pre


L3st3:

lds		r16,$0072
cp		r16,r20
brne	L3st6

rjmp	L3st5


L3st6:

lds		r16,$0074
cp		r16,r20
brne	L3st8

rjmp	L3st7


L3st8:

lds		r16,$0076
cp		r16,r20
brne	L3st10

rjmp	L3st9


L3st10:

lds		r16,$0086
cp		r16,r20
brne	L3st12

rjmp	L3st11


L3st12:

rcall	Delay100	
rcall	Delay30
rjmp	Loop4pre


L3st5:		;relay 1

lds		R16,$0073		;relay 1 TOGGLE
ori		r16,4		;TOGGLE bit
sts		$0073,r16


InsertL3a:

sbrc	r17,0		;skip if r17,0(1) is clr (relay 1 is off)
rjmp	InsertL3a1

ldi		r16,1		;put relay 1 in $006D
sts		$006D,r16		;set beep ON for relay 1

rcall	Delay100
rcall	Delay34
rjmp	Loop4pre


InsertL3a1:

rcall	Delay100
rcall	Delay36
rjmp	Loop4pre


L3st7:		;relay 2

lds		r16,$0075
ori		r16,4		;Relay 2 TOGGLE
sts		$0075,r16


InsertL3b:

sbrc	r17,1		;skip if r17,1(2) is clr (relay 2 is off)
rjmp	InsertL3b1

ldi		r16,2
sts		$006D,r16		;put relay 2 into $006D (set beep ON)

rcall	Delay100
rcall	Delay29
rjmp	Loop4pre


InsertL3b1:

rcall	Delay100
rcall	Delay31
rjmp	Loop4pre


L3st9:		;relay 3

lds		r16,$0077
ori		r16,4		;Relay 3 TOGGLE
sts		$0077,r16


InsertL3c:

sbrc	r17,2		;skip if r17,2(4) is clr (relay 3 is off)
rjmp	InsertL3c1

ldi		r16,3		;relay 3
sts		$006D,r16		;put 3 into $006D (set beep ON)

rcall	Delay100
rcall	Delay24
rjmp	Loop4pre


InsertL3c1:

rcall	Delay100
rcall	Delay26
rjmp	Loop4pre


L3st11:		;relay 4

lds		r16,$0087
ori		r16,4		;set relay 4 Toggle bit
sts		$0087,r16


Insert3d:

sbrc	r17,3		;skip if r17,3(8) is clr (relay 4 is off)
rjmp	InsertL3d1

ldi		r16,4		;relay 4
sts		$006D,r16		;put 4 into $006D (set beep ON)

rcall	Delay100
rcall	Delay19
rjmp	Loop4pre


InsertL3d1:

rcall	Delay100
rcall	Delay21
rjmp	Loop4pre


;--------------------------------------------

Loop4pre:

cbr	 	r18,1		;clear r18,0(1) turn send bit off
sbis	PIND,2		;skip if T.P. not touched
rjmp	LoopExit

sbrs	r19,3		;skip if r19,3(8) set (master)
rjmp	L4P1c
rjmp	L4P1b


L4P1b:

rcall	QuickInputsCheck  ;(23 clock cycles)
rjmp	L4P1d


L4P1c:

rcall	Delay24
rjmp	L4P1d


L4P1d:

cpi		r20,0		
breq	L4P2		;branch if it is command 0 data send
rjmp	L4P1		;units 1 on up


L4P2:

sbrs	r19,3		;skip if r19,3(8) set (master)
rjmp	L4P3


L4P4:				;master unit branch

lds		r16,$0083		;load in command 0 send data
sbrs	r16,3		;skip if slot 4 data is set
rjmp	L4P5		;no pulse send
rjmp	L4P6		;send pulse


L4P5:				;No pulse send

rcall	Delay100
rcall	Delay17
rjmp	Loop4post


L4P6:				;command 0 pulse send

sbr		r18,1		;set r18,0(1) for pulse send
cbr		r16,8		;clear $0083,3(8)
sts		$0083,r16
rcall	Delay100
rcall	Delay12
rjmp	Loop4post


L4P3:				;slave - no pulse send

rcall	Delay100
rcall	Delay21
rjmp	Loop4post


L4P1:				;all other units except unit 0

lds		r16,$006D		;load received relay number if set
cpi		r16,0
brne	L4P1e		;send pulse - relay number in data is off -- set for beep sounding
rjmp	L4P1f		;no pulse send


L4P1e:				;send pulse

sbr		r18,1		;set r18,0(1) for pulse send

sts		$006E,r16		;store beep/relay number data (1-4) into $006E for end of loop processing
clr		r16
sts		$006D,r16		;clear beep/relay data for this slot

rcall	Delay100
rcall	Delay12
rjmp	Loop4post


L4P1f:				;no pulse send

rcall	Delay100
rcall	Delay17
rjmp	Loop4post


;---------------------------------------------------------

Loop4post:

rcall	SinglePulse

sbrs	r18,1		;skip if r18,1(2) set (pulse received)
rjmp	L4st1


L4st6:

sbrs	r17,5		;skip if r17,5(32) is set - pre beep flag
rjmp	L4st6a

sbr		r19,2		;set r19,1(2) - beep flag
cbr		r17,32		;clear pre beep flag
rjmp	L4st2


L4st6a:

nop
nop
nop


L4st2:

cpi		r20,0
breq	L4st3
rjmp	L4st4


L4st3:

sbrs	r19,3		;skip if r19,3(8) is set (master)
rjmp	L4st5
rcall	Delay100
rcall	Delay46		;(146 clock cycles)
rjmp	Loop5pre


L4st5:

lds		r16,$0071		;get zero command
ori		r16,8		;set bit 3(8) 
sts		$0071,r16
rcall	Delay100
rcall	Delay40
rjmp	Loop5pre


L4st4:

rcall	Delay100
rcall	Delay47
rjmp	Loop5pre


L4st1:

cbr		r17,32		;clear pre beep flat r17,5(32)
rcall	Delay100
rcall	Delay55		;(155 clock cycles)
rjmp	Loop5pre


;--------------------------------------------

Loop5pre:

cbr		r18,1 		;clear r18,0(1) send bit

sbis	PIND,2		;skip if TP not touched 
rjmp	LoopExit	;jump out for touch plate decoding

sbrs	r19,3		;skip if r19,3(8) (master)
rjmp	L5P1c		;slave
rjmp	L5P1b		;master


L5p1b:

rcall	QuickInputsCheck	;(21 clock cycles)
rjmp	L5P1d


L5P1c:

rcall	Delay24
rjmp	L5P1d


L5P1d:

cpi		r20,0
breq	L5P2
rjmp	L5P1


L5P2:

sbrs	r19,3	;skip if r19,3(8) is set Master
rjmp	L5P3


L5P4:			;master

lds		r24,$0083	;load zero command
sbrs	r24,4	;skip if r24,4(16) is set
rjmp	L5P5
rjmp	L5P6


L5P6:		;send pulse

sbr		r18,1		;set r18,0(1) for puslse send
cbr		r24,16		;clear $0083,4(16)
sts		$0083,r24	;store away

rcall	Delay100
rcall	Delay12
rjmp	Loop5post


L5P3:

rcall	Delay100
rcall	Delay21
rjmp	Loop5post


L5P1:

lds		r16,$0080	;get unit number from touch plate entry
cp		r16,r20
breq	L5P7
rjmp	L5P8


L5P7:

lds		r24,$0081	;command put touch plate command in send
sbrs	r24,4	;skip if r24,4(16) is set
rjmp	L5P9	;out
rjmp	L5P10


L5P10:

sbr		r18,1	;set send bit, r18,0(1)
rcall	Delay100
rcall	Delay11
rjmp	Loop5post


L5P5:

rcall	Delay100
rcall	Delay17
rjmp	Loop5post


L5P8:

rcall	Delay100
rcall	Delay17
rjmp	Loop5post


L5P9:

rcall	Delay100
rcall	Delay13
rjmp	Loop5post


;---------------------------------------------------


Loop5post:

rcall	SinglePulse

sbrs	r18,1		;skip if r18,1(2), pulse received is set
rjmp	L5st1


L5st2:

cpi		r20,0		;zero command
brne	L5st3
sbrs	r19,3		;skip if r19,3(8) set (master)
rjmp	L5st3a
rjmp	L5st3b


L5st3a:				;slave

lds		r16,$0071
ori		r16,16		;set bit 4(16)
sts		$0071,r16
rjmp	L5st4


L5st3b:

rcall	Delay100
rcall	Delay45
rjmp	L5stEnd		;161-6=155


L5st3:

lds		r16,$0072		;Unit xx, Relay 1
cp		r16,r20
brne	L5st5

lds		r16,$0073		;get data byte
ori		r16,16		;set bit 4(16)
sts		$0073,r16
rjmp	L5st6


L5st4:

rcall	Delay100
rcall	Delay39
rjmp	L5stEnd		;161-6=155


L5st5:

lds		r16,$0074
cp		r16,r20		;Unit xx, Relay 2
brne	L5st7

lds		r16,$0075		;grab data byte
ori		r16,16		;set bit 4(16)
sts		$0075,r16
rjmp	L5st8


L5st6:

rcall	Delay100
rcall	Delay37
rjmp	L5stEnd		;161-6=155


L5st7:

lds		r16,$0076
cp		r16,r20		;Unit xx, Relay 3
brne	L5st9

lds		r16,$0077
ori		r16,16		;set bit 4(16)
sts		$0077,r16
rjmp	L5st10


L5st8:

rcall	Delay100
rcall	Delay32
rjmp	L5stEnd		;161-6=155


L5st9:

lds		r16,$0086
cp		r16,r20			;Unit xx, Relay 4
brne	L5st11

lds		r16,$0087
ori		r16,16		;set bit 4(16)
sts		$0087,r16
rjmp	L5st12


L5st10:

rcall	Delay100
rcall	Delay27
rjmp	L5stEnd		;161-6=155


L5st11:

rcall	Delay100
rcall	Delay28
rjmp	L5stEnd		;161-6=155


L5st12:

rcall	Delay100
rcall	Delay22
rjmp	L5stEnd		;161-6=155


L5st13:

clr		r24
rjmp	Loop1pre


L5st1:

rcall	Delay100
rcall	delay50
rjmp	L5stEnd		;161-6=155


L5stEnd:		;155 clock cycles to this point if returning to loop - 161-6=155

inc		r20
brne	L5st13
rjmp	LoopExit1



;-----------------------------------------------

LoopExit1:

DecodeRelay:

lds		r16,$0073		;relay 1
tst		r16
breq	DR1

rjmp	Relay1Mode	


DR1:

lds		r16,$0075
tst		r16
breq	DR2

rjmp	Relay2Mode	


DR2:

lds		r16,$0077
tst		r16
breq	DR3

rjmp	Relay3Mode


DR3:

lds		r16,$0087
tst		r16
breq	DREnd

rjmp	Relay4Mode


DREnd:

;This section executes local relays 1 to 4 commands

lds		r16,$0080		;get unit number
cpi		r16,1
brne	DRE2a

lds		r16,$0081		;get command
sbrs	r16,0		;skip if $0081,bit 0(1) is set - ON command
rjmp	DRE5d

rcall	ONRelay1
rjmp	DREnd2


DRE5d:

sbrs	r16,1		;skip if $0081 bit 1(2) is set - OFF command
rjmp	DRE5e

rcall	OFFRelay1
rjmp	DREnd2


DRE5e:

sbrs	r16,2		;skiip if $0081 bit 2(4) is set - TOGGLE command
rjmp	DREnd2

rcall	TOGGLERelay1
rjmp	DREnd2


;----

DRE2a:				;Check Relay 2 for command

lds		R16,$0080
cpi		r16,2		
brne	DRE3a

lds		r16,$0081		;get command for relay #2
sbrs	r16,0		;skip if $0081 bit 0(1) is set - ON command
rjmp	DRE6d

rcall	ONRelay2
rjmp	DREnd2


DRE6d:

sbrs	r16,1		;skip if $0081 bit 1(2) is set - OFF command
rjmp	DRE6e

rcall	OFFRelay2
rjmp	DREnd2


DRE6e:

sbrs	r16,2		;skip if $0081 bit 2(4) is set - TOGGLE command
rjmp	DREnd2


rcall	ToggleRelay2
rjmp	DREnd2


;---------

DRE3a:

lds		r16,$0080		;get unit number
cpi		r16,3
brne	DRE4a

lds		r16,$0081		;get relay 3 command
sbrs	r16,0		;skip if $0081 bit 0(1) is set - ON command
rjmp	DRE7d

rcall	OnRelay3
rjmp	DREnd2


DRE7d:

sbrs	r16,1		;skip if $0081 bit 1(2) is set - OFF command
rjmp	DRE7e

rcall	OffRelay3
rjmp	DREnd2


DRE7e:

sbrs	r16,2		;skip if $0081 bit 2(4) is set - TOGGLE command
rjmp	DREnd2

rcall	ToggleRelay3
rjmp	DREnd2


;-------

DRE4a:

lds		r16,$0080		;get unit number
cpi		r16,4
brne	DREnd2

lds		r16,$0081		;get relay #4 command
sbrs	r16,0		;skip if $0081 bit 0(1) is set - ON command
rjmp	DRE8d

rcall	OnRelay4	
rjmp	DREnd2


DRE8d:

sbrs	r16,1		;skip if $0081 bit 1(2) is set - OFF command
rjmp	DRE8e

rcall	OffRelay4
rjmp	DREnd2


DRE8e:

sbrs	r16,2		;skip if $0081 bit 2(4) is set - TOGGLE command
rjmp	DREnd2

rcall	ToggleRelay4
rjmp	DREnd2



DREnd2:

rcall	ClearRelayData	;clear $0073,5,7, $0080, $0081, $0087 send data

sbrs	r19,1			;skip if r19,1(2) is set - beep flag
rjmp	LoopExit2

sbi		PortB,3		;ON sonalert -- beep sonalert -- relay addressed is off
rcall	Delay150ms
cbi		PortB,3		;OFF sonalert
cbr		r19,2		;clear r19,1(2) beep flag


LoopExit2:

clr		r16
sts		$006E,r16		;zero beep contents ($006E not used 3/10/2009 - remnant code)
rjmp	LoopExit



; ----   DECODE RELAY - RELAY MODES
;----------------------------------

;  RELAY MODES: ON, OFF, TOGGLE -- r16 contains bits 0, 1, 2 for commands


Relay1Mode:		;Relay1

cpi		r16,1	;is it ON command?
brne	R1M1

rcall	OnRelay1
rjmp	DREnd


R1M1:

cpi		r16,2	;is it OFF command
brne	R1M2

rcall	OffRelay1
rjmp	DREnd

 
R1M2:

cpi		r16,4	;is it Toggle command
brne	R1M3

rcall	ToggleRelay1
rjmp	DREnd


R1M3:

rjmp	DREnd

;----------------------


Relay2Mode:		;Relay No. 2

cpi		r16,1	;is it ON command?
brne	R2M1

rcall	OnRelay2
rjmp	DREnd


R2M1:

cpi		r16,2	;is it OFF command
brne	R2M2

rcall	OffRelay2
rjmp	DREnd


R2M2:

cpi		r16,4	;is it Toggle command
brne	R2M3

rcall	ToggleRelay2
rjmp	DREnd


R2M3:

rjmp	DREnd


;-------------------------

Relay3Mode:

cpi		r16,1	;is it ON command?
brne	R3M1

rcall	OnRelay3
rjmp	DREnd


R3M1:

cpi		r16,2	;is it OFF command
brne	R3M2

rcall	OffRelay3
rjmp	DREnd


R3M2:

cpi		r16,4	;is it Toggle command
brne	R3M3

rcall	ToggleRelay3
rjmp	DREnd


R3M3:

rjmp	DREnd


;-----------------------

Relay4Mode:

cpi		r16,1	;is it ON command?
brne	R4M1

rcall	OnRelay4
rjmp	DREnd


R4M1:

cpi		r16,2	;is it OFF command
brne	R4M2

rcall	OffRelay4
rjmp	DREnd


R4M2:

cpi		r16,4	;is it Toggle command
brne	R4M3

rcall	ToggleRelay4
rjmp	DREnd


R4M3:

rjmp	DREnd

; END RELAY MODES


;-------------------------------------------------------

LoopExit:

pop		r25
pop		r24
pop		r23
pop		r20


;------------------------------

Relay1AutoOff:

push	r30
push	r31

lds		r30,$007A
lds		r31,$007B
mov		r16,r30
or		r16,r31

breq	R1AO1

sbiw	r30,1		;subtrack one from Z register pair, r30,r31
					;   each second
sts		$007A,r30
sts		$007B,r31	;store back

brne	R1AO1

rcall	OffRelay1	;off relay, Count down stored in $007A, $007B
					; is now fulfilled

R1AO1:

pop		r31
pop		r30


Relay1AutoOffEnd:
;----------------------------------


;--------------
;SPLIT HERE FOR EITHER SLAVE OR MASTER ROUTINE CALLS
;--------------


;----------------------------------


ProcessesStart:

sbrs	r19,3			;skip if r19,3(8) is set (Master)
rjmp	SlaveRoutines	;got to slave routines

;-------------------------


; MASTER ROUTINES


;-------------------------

; 4 a.m. correction consists of three sections.  (1)  Pulse4amDOW causes master to pulse out the DOW at 4:00 a.m correcton. 
;  Example:  3 pulses = Tuesday, 1 pulse = Sunday, etc.   (2) L4AM1  Starts up 10 second window to receive 2nd pulse (if there)
;  from QuickInputsCheck-4 a.m. and also sets 4 a.m. correcton for master unit clock.  (3) L4AM2 determines if a 2nd pulse is,
;  in fact, received, setting up day of week count in R9 for use by Pulse4amDOW.


Pulse4amDOW:		;Pulse is sent out number of times in r9 (Day of week counter)

tst		r9			;is r9, 4am DOW pulse count zero?
breq	PulseDOWEnd

dec		r9			;decrement 4am DOW pulse count
lds		r16,$0083
sbr		r16,1		;set $0083,0(1) for pulse send in next round
sts		$0083,r16	;store change back in $0083

PulseDOWEnd:
;----------------------------------------------
L4AM1:				;If atomic Clock pulse is detected and Delay Counter $00FB is zero
					; then Delay Counter is set to 10 and 4 a.m. is set in SRAM

lds		r16,$00FB	;get count from 10 counter
tst		r16			;Check 10 sec delay counter
brne	L4AM1end

sbrs	r29,0		;skip if r29,0(1), 4am QIC flag is set
rjmp	L4AM1end

cbr		r29,1		;clear r29,0(1), 4am QIC flag
ldi		r16,10
sts		$00FB,r16	;set count of 10 in 10 Delay counter

ldi		r16,240		
sts		$0068,r16	;put 240 (4 a.m. time) in LSB
clr		r16
sts		$0069,r16	;clear high time byte
sts		$006B,r16	;zero seconds count


L4AM1end:
;-------------------------------------------------

L4AM2:				;If 10 count ($00FB) is zero, exit, otherwise if 10 count = 9
					; reset Quick Inputs Check bit else if 10 count =less than 9
					; then check for 2nd pulse (Sunday).  If it is Sunday (2nd pulse)
					; then set Day of Week to 1 (Sunday).
					;Also puts value in r9 for day of week number of pulses to send.

lds		r16,$00FB	;load count in $00FB, 10 second delay window counter
tst		r16
breq	L4AM2end	;jump out - no action needed

dec		r16
sts		$00FB,r16	;store decremented count

cpi		r16,9
breq	L4AM2end1	;no action needed		;

cpi		r16,8
breq	L4AM2end1	;no action needed - clear QIC flag

cpi		r16,0
breq	L4AM2b		;set r9 with DOW count from $006A

sbrs	r29,0		;skip if r29,0(1), QIC flag is set
rjmp	L4AM2end

ldi		r16,1
sts		$006A,r16	;set Sunday (1) into $006A, day of week counter for master

rjmp	L4AM2end


L4AM2b:				;reset QIC, 4am flag and set DOW ($006A) to 1 (Sunday)

cbr		r29,1		;clear r29,0(1), 4am QIC flag
lds		r9,$006A	;Put DOW value in $006A into r9


L4AM2end1:

cbr		r29,1		;reset QIC, 4am flag, r29,0(1)


L4AM2end:
;----------------------------------------
ProxProcess:		;Checks quick inputs a assures only one pulse is sent 
					; to remote units during 10 second countdown

lds		r16,$0083		;get Send register
cbr		r16,2		;clear proximity detector SEND bit
sts		$0083,r16

sbrs	r29,1		;skip if r29,1(2) is set - proximity detector flag
					; quick inputs check
rjmp	ProxPEnd

tst		r11			;check count in r11
brne	ProxP1

lds		r16,$0083
sbr		r16,2		;Set $0083,1(2) - prox detector send bit
sts		$0083,r16

ldi		r16,10		;put count of 10 into r11
mov		r11,r16


ProxP1:

cbr		r29,2		;clear flag r29,1(2) - Proximity Detector flag


ProxPEnd:
;----------------------------------------
MasterDayNightProcess:

;$0071	has two for dusk, one for dawn

;$0083 0(1) = 4 a.m.
;	   1(2) = Proximity
;      2(4) = Dusk/Dawn (dawn = bit 2(4) ONLY)
;      3(8) = Burglar Alarm
;      4(16) = Sunday / Dusk-Dawn (dusk = bit 2(4) + bit 4(16))


lds		r16,$006B		;load seconds counter
cpi		r16,5			;FIVE seconds into time - turn on power to CDS photocell
						;  five seconds into count
breq	DDPS3

cpi		r16,6			;SIX second into time - make check on CDS photocell
						;  six seconds into count
breq	DDPS1

rjmp	DDPSEnd2


DDPS3:

sbi		PortD,7		;on power to CDS day/night sensing cell
rjmp	DDPSEnd2


DDPS1:				;six seconds in and start processing

push	r20			;temp dawn minute count
push	r28			;temp dusk minute count

lds		r20,$0078	;dawn minute count
lds		r28,$0079	;dusk minute count

sbis	PIND,1		;skip if PIND,0 is set (Dusk/Dawn status) 1 = night
rjmp	DDPS2		;it is daytime


DDPS4:				;it is night time

ldi		r20,15		;reset r20, dawn counter to 15 minute count 
sts		$0078,r20	;store into dawn count

tst		r28			;dusk count
breq	DDPSEnd

dec		r28			;decrement dusk count by 1 minute
sts		$0079,r28
brne	DDPSEnd		;no action ... go on

ldi		r16,2		;two pulses indicate a dusk condition
mov		r14,r16		;put two pulses into r14 (dusk/dawn pulse count)

rjmp	DDPSEnd


DDPS2:				;it is day time

ldi		r28,15		;reset r28, dusk counter to 15 minute count
sts		$0079,r28	;store value back in SRAM

tst		r20
breq	DDPSEnd

dec		r20			;decrement dawn count by 1 minute
sts		$0078,r20
brne	DDPSEnd		;no action .... go on

ldi		r16,1		;ONE pulse indicates a daylight condition
mov		r14,r16		;put one pulse count into r14 (dusk/dawn pulse count)

rjmp	DDPSEnd


DDPSEnd:

cbi		PortD,7		;off CDs power
pop		r28
pop		r20


DDPSEnd2:
;-----------------------------------------------------------
DuskDawnPulseSend:	;Sends out one pulse for dawn, two for dusk

tst		r14			;r14 = number of pulses count
					; 2 pulses for dusk, one for dawn
breq	PSend

lds		r16,$0083
sbr		r16,4		;set dusk/dawn bit in $0083,2(4)
sts		$0083,r16	;put back in $0083
dec		r14			;decr r14, number of pulses to be sent


PSend:
;----------------------------------
BurglarAlarmProcess:	;limits sending of Unit 0 pulse to every 10 seconds

lds		r16,$0083		;get send register
cbr		r16,8		;clear $0083,3(8) send flag bit
sts		$0083,r16

sbrs	r29,3		;skip if r29,3(8) is set (Burglar Alarm flag)
rjmp	BAPEnd

tst		r12			;r12 is counter for burglar alarm pause
brne	BAP1

lds		r16,$0083
sbr		r16,8		;set send bit in send register $0083,3(8)
sts		$0083,r16

ldi		r16,10		;10 second pause before detecting new puls
mov		r12,r16		;put in pause count register r12


BAP1:

cbr		r29,8		;clear r29,3(8) burglar alarm flag


BAPEnd:
;--------------------------------------
Decrement4amReset:		;counts down r7, the 4am pause counter

tst		r7
brne	D4am1
rjmp	d4amEnd


D4am1:

dec		r7
brne	D4amEnd


D4amEnd:
;-----------------------
DecrementProxReset:

tst		r11			;pause counter for Proximity Detector, r11
brne	DPR1
rjmp	DPREnd


DPR1:

dec		r11


DPREnd:
;----------------------------
DecrementBurgAlarmReset:

tst		r12			;pause counter for Burg. Alarm, r12
brne	DBAR1
rjmp	DBAREnd


DBAR1:
dec		r12


DBAREnd:


;--------------------------------

sbis	PIND,2			;skip if touch plate not touched
rjmp	DecodeButtonPush

rcall	QuickInputsCheck
rjmp	MasterLoop


;----------------------------------------------

SLAVEroutines:

;----------------------------------------------


; SLAVE SERVICE ROUTINES FOLLOW HERE


;-----------------------------------------------


DuskDawnPulseReceived:		;Sets $0084 with one (dusk) or two (dawn) when detected
							;r0 = gate count,    r14 = pulses received

tst		r0			;test gate count of 10 sec
brne	PR2


PR1:

lds		r16,$0071
sbrs	r16,2		;skip if $0071,2(4) is set, dusk/dawn
rjmp	PRend

cbr		r16,4		;clear $0071 bit received
sts		$0071,r16

inc		r14			;incr pulse received count by one - r14

ldi		r16,12		
mov		r0,r16		;put 12 second gate count in r0, gate counter

rjmp	PR6


PR2:

mov		r16,r0		;move gate count into r16 for processing
cpi		r16,1
breq	PR3


cpi		r16,2
breq	PR4

lds		r16,$0071	;get dusk/dawn $00712,(4)
sbrc	r16,2
rjmp	PR5


PR6:

dec		r0			;decrement gate count
rjmp	PRend


PR3:

clr		r16
sts		$0084,r16	;clear out data in $0084
mov		r14,r16		;zero pulses received count
rjmp	PR6


PR4:

sts		$0084,r14	;put pulses received count into $0084
rjmp	PR6


PR5:

lds		r16,$0071	
cbr		r16,4		
sts		$0071,r16	;clear dusk/dawn bit, $0071,2(4)

inc		r14			;increment pulse count
rjmp	PR6


PRend:
;----------------------------------------------
Receive4am:			;receives 4 a.m. pulse in $0071 and resets clock to 4 a.m. and also sets Day of Week
					;  R8 is gate counter    R9 is day of week pulse counter

tst		r8			;test Gate Counter, r8, for zero
breq	R4a

mov		r16,r8		;move into r16 for processing
cpi		r16,1
breq	R4b

cpi		r16,2
breq	R4c

lds		r16,$0071
sbrc	r16,0		;skip if $0071,0(1), 4a.m. bit clear
rjmp	R4d


R4e:

dec		r8			;decrement gate counter r8

rjmp	R4end


R4a:

lds		r16,$0071
sbrs	r16,0		;skip if $0071,0(1) set (4 a.m. bit)
rjmp	R4end


cbr		r16,1		;clear out $0071,0(1) 4 a.m. bit
sts		$0071,r16	;store away

inc		r9			;inc r9, DOW pulse counter

ldi		r16,12
mov		r8,r16		;put count of 12 into r8 gate counter
					;  12 second gate
ldi		r16,240
sts		$0068,r16	;put in 4:00:00 a.m. time into $0068 low byte
					; gives some lead time to compensate for touch plate

clr		r16
sts		$0069,r16	;zero for high byte
sts		$006B,r16	;put zero into seconds counter

rjmp	R4e


R4b:

clr		r9			;clear count in r9 pulse counter
rjmp	R4e


R4c:

sts		$006A,r9	;put R9 pulse count (DOW)into $006A - DOW byte
					; 1=Sun, 2=Mon, 3=Tue, 4=Wed, 5=Thr, 6=Fri, 7=Sat
rjmp	R4e


R4d:

lds		r16,$0071
cbr		r16,1		;clear $0071,0(1) 4 a.m. bit
sts		$0071,r16	;store away

inc		r9			;increment pulse count by one
rjmp	R4e


R4end:
;--------------------------------------
LivedInON:			;turns on relay 1 if lived-in function enabled

sbrs	r17,6		;skip if r17,6(64), lived in enable bit set
rjmp	LINOnEnd

lds		r16,$0084	;load Pulse Store number into r16
cpi		r16,2		;dusk command - two pulses received
brne	LINOnEnd

rcall	OnRelay1	;turn on relay 1
clr		r16
sts		$0084,r16	;clear $0084 dusk/dawn pulse store


LINOnEnd:
;------------------------------
LivedInOff:			;puts in random countdown number into $0082 at time
					;  stored in $0088,89.  This will initiate turn off
					;  of relay 1 when count reaches zero in Lived In Off Counter.

sbrs	r17,6			;skip if r17,6(64) Lived In is active 
rjmp	LINOffEnd2

lds		r16,$006B		;load seconds counter
cpi		r16,40			;make this check at 40 seconds into minute
brne	LINOffEnd2

push	r30

lds		r30,$0089		;load high programmed time match byte
lds		r16,$0069		;present time high byte
cp		r30,r16
brne	LINOffEnd

lds		r30,$0088		;load LSD programmed time match
lds		r16,$0068		;present time low byte
cp		r30,r16
brne	LINOffEnd

tst		r10				;grab present random number in r10
breq	LINOffEnd3
sts		$0082,r16		;store in $0082, Lived In Random Count Down counter
rjmp	LINOffEnd


LINOffEnd3:

inc		r10				;to assure a non zero for a random number (0 - 4.2 minutes)
sts		$0082,r10		;store in $0082, Lived In Random Count Down counter


LINOffEnd:

pop		r30


LINOffEnd2:
;--------------------------------
LINOffCounter:			;turns off relay when count in $0082 reaches zero
						;  Lived In Off Relay when count reaches zero

lds		r16,$0082
tst		r16
breq	LINOffCounterEnd

dec		r16
sts		$0082,r16

brne	LINOffCounterEnd

rcall	OffRelay1		;off relay 1


LINOffCounterEnd:
;--------------------------------------------
LivedIn2Initialize:		;when double pulse is received in $0084 and 
						;  Lived In 2 flat is set, it starts process

sbrs	r18,3			;skip if r18,3(8) lived in 2 enabled is set
rjmp	LIn2InitEnd

lds		r16,$0084		;load dusk/dawn pulse store (1 (dawn) or 2(dusk)) when received
cpi		r16,2			
breq	LIn2Init1

rjmp	LIn2InitEnd


LIn2Init1:

clr		r16
sts		$0084,r16		;clear $0084, dusk/dawn pulse store
sts		$00F3,r16		;clear high byte of OFF counter

sbr		r18,16			;set r18,4(16) bit to activate lived in 2 "in process" flag

in		r16,TCNT1L		;grab random number from counter 1 low byte

sts		$00F2,r16		;put random number into $00F2,F3 OFF counter
						;keeps light off until random count is reached
						;varies off time up to 4.2 minutes


LIn2InitEnd:
;---------------------------------------
LIn2OffDelay:			;turns Lived In 2, relay 1 on at end of delay
						; and activate delay for relay on cycle
						; only active when count is in $00F2, $00F3
						;puts new value into lived in 2 ON counter

sbrs	r18,4			;skip if Lived In 2 -in process- enable flag is set, r18,4(16)
rjmp	LIn2OffEnd2

push	r30
push	r31

lds		r30,$00F2		;low byte of OFF counter
mov		r16,r30			;copy in r16

lds		r31,$00F3		;high byte of OFF counter

or		r16,r31	
breq	LIN2OffEnd		;no action

sbiw	r30,1			;subtract 1 from register z pair, r30,31
sts		$00F2,r30
sts		$00F3,r31		;store new number back
brne	LIN2OffEnd

;Put new number into Lived In 2 ON counter to continue process

ldi		r31,2			;put in minimum of 10 minutes on
ldi		r30,88
lds		r16,$00FF		;9X times random number stored in $00FF
						;  

LIN2a:

add		r30,r10			;add in random seconds into r30 from r10
brcc	LIN2c

inc		r31				;inc r31 if overflow


LIN2c:

dec		r16
brne	LIN2a			;loop back to complete multiplication

sts		$00F0,r30		;put value into lived in 2 ON counter
sts		$00F1,r31

rcall	OnRelay1		;turn on relay 1		


LIn2OffEnd:

pop		r31
pop		r30


LIn2OffEnd2:
;------------------------------------
LIn2OnDelay:			;relay turns off after delay in $00F0,$00F1 goes to zero
						; operation continues when new value for off count is put
						; into $00F2,$00F3

sbrs	r18,4			;skip if Lived In 2 -in process- enable flag is set, r18,4(16)
rjmp	LIn2OnEnd2

push	r30
push	r31

lds		r30,$00F0
mov		r16,r30
lds		r31,$00F1		;get ON counter count, if any in $00F0,$00F1

or		r16,r31
breq	LIn2OnEnd		;no action needed

sbiw	r30,1			;subtract 1 from count
sts		$00F0,r30
sts		$00F1,r31		;put new count back
brne	LIn2OnEnd

ldi		r31,2			;put in minimum of about 10 minutes OFF delay
ldi		r30,88
lds		r16,$00FE		;number of loops from $00FE store area


LIn2k:

add		r30,r10			;add random number to r30
brcc	LIn2l

inc		r31				;inc R31 due to carry


LIn2l:

dec		r16
brne	LIn2k

sts		$00F2,r30
sts		$00F3,r31		;put new off time for light in $00F2,F3

rcall	OffRelay1


LIn2OnEnd:

pop		r31
pop		r30

LIn2OnEnd2:
;-----------------------------------
OffLivedIn2InProcess:

sbrs	r18,4			;skip if r18,4(16), InProcess flag is set
rjmp	OLIn2end2

push	r30
push	r31

lds		r30,$0088
lds		r31,$0089		;load time to turn lived in operations off for night

lds		r16,$0068
cp		r16,r30
brne	OLIn2end

lds		r16,$0069
cp		r16,r31
brne	OLIn2end

;it is a time match

cbr		r18,16			;clear r18,4(16) Turn OFF lived in 2 in process flag

tst		r10
breq	OLIn2IP1

sts		$00F4,r10			;store random off of 4.2 minutes into $00F4, Final 
						;  Lived In 2 Off countdown
rjmp	OLIn2end


OLIn2IP1:

inc		r10
sts		$00F4,r10			;store away random countdown number for off of light in $00F4
							;  for use by FinalLivedIn2Off 

OLIn2end:

pop		r31
pop		r30


OLIn2end2:
;-------------------------------
FinalLivedIn2Off:		;turns off relay 1 when in process flag is off and
						; random count in $00F4 is zero

lds		r16,$00F4		;get count in $00F4, Lived In 2 final countdown off count
tst		r16
breq	FLI2end			;ignore - already zero

dec		r16
sts		$00F4,r16
brne	FLI2end

rcall	OffRelay1


FLI2end:
;---------------------------------
BurglarAlarmLightOn:		;If burglar alarm flag is set and command zero has trigger for 
							; burglar alarm, Relay 1 light comes on

sbrs	r18,7		;skip if r18,7(128) is set (Burglar Alarm Enabled Flag high)
rjmp	BALOend		

lds		r16,$0071	;load unit zero commands
cpi		r16,8		;$0071 3(8)
brne	BALOend

push	r28
push	r30
push	r31

ldi		r30,$29		;EEPROM store location
clr		r31

rcall	fetchEEPROM
mov		r28,r16		;move LSB into r28 temporarily
inc		r30
rcall	fetchEEPROM

sts		$007C,r28
sts		$007D,r16	;move seconds data from EEPROM into SRAM $007C,$007D
					; (on time for light)

rcall	OnRelay1

pop		r31
pop		r30
pop		r28


BALOend:
;---------------------------------
OffBurglarAlarmRelay1:	;Relay 1 turned off when seconds count in $007C,$007D is zero
						; Original count stored in EEPROM at $0029,$002A

sbrs	r18,7			;skip if r18,7(128) set (burglar alarm flag)
rjmp	OBAR1end

push	r30
push	r31

lds		r30,$007C
lds		r31,$007D		;B.A. countdown count SRAM store area
mov		r16,r31

or		r16,r30
breq	OBAR1end1	;jump out if seconds count is zero

sbiw	r30,1		;decrement register pair z by one
sts		$007C,r30
sts		$007D,r31
brne	OBAR1end1

rcall	OffRelay1


OBAR1end1:

pop		r31
pop		r30


OBAR1end:	
;------------------------------------
ProximityLightOn:		;If proximity alarm flag is set and command zero has trigger for 
							; burglar alarm, Relay light one comes on

sbrs	r29,6		;skip if r29,6(64) is set (Proximity Flag high)
rjmp	PLOend		

lds		r16,$0071		;load unit zero commands
cpi		r16,2			;$0071 (two pulses will activate)
brne	PLOend

push	r28
push	r30
push	r31

ldi		r30,$2B		;Proximity EEPROM store location
clr		r31

rcall	fetchEEPROM
mov		r28,r16		;move LSB into r28 temporarily
inc		r30
rcall	fetchEEPROM

sts		$007E,r28
sts		$007F,r16		;move seconds data from EEPROM into SRAM $007E,$007F
					; (on time for light when proximity tripped)
rcall	OnRelay1

pop		r31
pop		r30
pop		r28


PLOend:
;---------------------------------
OffProximityRelay1:	;Relay 1 turned off when seconds count in $007E,$007F is zero
						; Original count stored in EEPROM at $0029,$002A

sbrs	r29,6		;skip if r29,6(64) is set - proximity flag active
rjmp	OPR1end

push	r30
push	r31

lds		r30,$007E
lds		r31,$007F		;B.A. countdown count SRAM store area
mov		r16,r31

or		r16,r30
breq	OPR1end1	;jump out if seconds count is zero

sbiw	r30,1		;subtract one from pair z 
sts		$007E,r30
sts		$007F,r31

brne	OPR1end1

rcall	OffRelay1


OPR1end1:

pop		r31
pop		r30


OPR1end:
;------------------------------------
NiteLiteON:			;turns on relay if both dusk
					;  and nite light flags are set.

sbrs	r17,7		;skip if r17,7(128) nite lite activated
rjmp	NLONEnd

lds		r16,$0084	;get receive data
cpi		r16,2		;$0084 has two pulses (Dusk)
brne	NLONEnd

rcall	OnRelay1
clr		r16
sts		$0084,r16	;clear $0084 contents


NLONEnd:
;-------------------------------
NLOff:				;turns off relay 1 

sbrs	r17,7		;skip if r17,7(128) nite lite activated
rjmp	NLOffEnd

lds		r16,$0084
cpi		r16,1		;$0084 has ONE pulse (dawn)
brne	NLOffEnd

rcall	OffRelay1
clr		r16
sts		$0084,r16	;clear dusk/dawn pulse count


NLOffEnd:
;---------------------------------


;--------------------------------


;END of SLAVE routines

;--------------------------------

cbr		r29,15			;clear r29,- quick inputs 4 a.m. flag, Prox flag,
						;  dusk/dawn flag, and burglar alarm flag

in		r16,TCNT1L		;grab number from counter 1 low byte
add		r10,r16			;add new random number to r10

clr		r16
sts		$0071,r16		;clear all data in receive location for Unit 0

rcall	TimeSubroutine	;In this instance (slave), this is NOT an interrupt but a simple
						; rcall to the Time Subroutine. It checks for alarm match and
						; steps clock.

sbis	PIND,2			;skip if touch plate not touched
rjmp	DecodeButtonPush

rjmp	SlaveLoop


;------------------------------------------------------------

StallButtonPush:	;stalls and waits for button push entry.

sei				;ON interrupts to keep timing during button push routines
;				     using crystal clock


SBP1:

sbic	PIND,2		;skip if PIND,2, touch plate is touched
rjmp	SBP1

rjmp	ButtonPush


;-----------------------------------------------------------		

ButtonPush:		;Takes in dit, dah, holddown, and coded input

sei				;ON interrupts -- to keep clock timing during button pushing routines

;	Dit = 1
;	Dah = 10
;	Hold down = 50

;	$00EA,r30		;LSD of total count
;	$00EB,r31		;MSD of total count

push	r0		;number of code digits
push	r1		;low code byte
push	r2		;next high code byte
push	r3		;next high code byte
push	r4		;highest code byte
push	r18		;dit count
push	r20		;holddown count
push	r23		;universal counter
push	r24		;combined count
push	r28		;dah count
push	r30		;has total count of entry in Z register, r30,31
push	r31

;r30,r31 (Z register pair contain total value of entry)

clr		r18		;Dit Count
clr		r20		;HoldDown count
clr		r23		;universal counter
clr		r28		;dah count
clr		r0		;clear number of bits in code
clr		r1		;clear lowest code count
clr		r2		;clear next code count
clr		r3		;clear next code count
clr		r4		;clear next code count (clear all code counts)


				;r30,31 has total count

B1:

inc		r10			;increment r10, random number
inc		r23			;universal counter
rcall	Delay40th
cpi		r23,60		;delay before first holddown count - 40=one second 
brsh	B3			;branch if - it is a holddown count

sbis	PIND,2		;skip if switch is NOT down
rjmp	B1


B2:

cpi		r23,20		;compare r23 for 20 (1/2 second dah count)
brlo	B2b			;branch if lower (dit count)


B2a:				;it is dah count

inc		r28			;increment dah count

sec					;set carry flag -- it is dah entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

cbi		PortB,1		;on RED LED
rcall	Delay40th
rcall	Delay40th	;delay for longer on of LEDs
rcall	Delay40th
rcall	Delay40th
sbi		PortB,1		;off LED
clr		r23
inc		r0			;increment number of code digits


B2d:				;no touch/switch up detection

sbis	PIND,2		;skip if switch is up
rjmp	B2e			;jump back - there is a new entry coming in

inc		r23
rcall	Delay40th
cpi		r23,70		;has switch been up 70 counts? 40=one second (1.75 seconds)
brsh	B4Decode1
rjmp	B2d


BPEnd1:

rjmp	BPEnd


B2e:

clr		r23
rjmp	B1


B2b:				;Dit count detected

inc		r18			;increment dit count

clc					;clear carry flag -- it is dit entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

cbi		PortB,1
rcall	Delay40th
sbi		PortB,1

inc		r0			;increment number of code digits (r0)
clr		r23			;clear r23
rjmp	B2d


B3:					;Holddown count has been detected

inc		r20			;increment holddown count
cbi		PortB,1
rcall	Delay40th
rcall	Delay40th
sbi		PortB,1		;flash RED LED

clr		r23			;reset r23 to zero count


B4:

inc		r23
rcall	Delay40th
sbic	PIND,2		;skip if switch is down
rjmp	B4a			;jump out and look for next entry

cpi		r23,30		;pause between continuous hold down of switch
brlo	B4

rjmp	B3


B4a:

clr		r23
rjmp	B2d			;Check for exit or new entry



B4Decode1:

;This section finalizes.  It combines counts

clr		r30
clr		r31			;ready registers for input

mov		r16,r28		;move dah count in r28 into r16


B4comb1:

tst		r16
breq	B4comb2

adiw 	r30,10		;add value of 10 in r30 to r314
dec		r16			
rjmp	B4comb1


B4comb2:

add		r30,r18		;add value in r18 (dit count) to r24
brcs	B4comb2a
rjmp	Bholdown1


B4comb2a:

inc		r31			;carry was set so increment higher byte


Bholdown1:

;Combined value now in Z register pair, r30,31

mov		r16,r20		;put hold down count into r16

Bholdown1b:

tst		r16			;test r20(r16), hold down count
breq	BPEnd
adiw	r30,50		;add fifty count to total
dec		r16
rjmp	Bholdown1b

;Complete combined value now in Z register pair


BPEnd:

;Store away in SRAM $00E7-00EB, Dit,Dah,HolddownCount,LSD Total, MSD Total

sts		$00E7,r18		;number of dits
sts		$00E8,r28		;number of dahs
sts		$00E9,r20		;number of hold down counts
sts		$00EA,r30		;LSD of total count
sts		$00EB,r31		;MSD of total count

sbiw	r30,1			;One Less for PLUS ONE entry system numbers, so as one can
						;  enter 255 as a valid number (256 as touch plate entry)

sts		$0100,r30		;Store away PLUS ONE entry in $0100 (one less than touched on plate)
sts		$0101,r31		;Store away high byte Plus One Entry in $0101

pop		r31
pop		r30
pop		r28
pop		r24
pop		r23
pop		r20
pop		r18
pop		r4
pop		r3
pop		r2
pop		r1
pop		r0

ret


;-------------------------------

DecodeButtonPush:  	;NOT A SUBROUTINE .... uses rjmp's

rcall	ButtonPush

push	r28			;Load LSD, MSD counts into r28,29
push	r29

lds		r28,$00EA		;get LSB of total count
lds		r29,$00EB		;get MSB of total count
lds		r16,$00E9		;get number of Hold Down touches

;following section decodes Touch Plate entries


DBP1a:

cpi		r16,7		;7 DAH touches (350) = Set every day Quick Set Alarm (0=no alarm)
brne	DBP1aa

rcall	QuickSetDisplayAlarm

rjmp	DBPend


DBP1aa:

cpi		r16,8		;8 DAH touches = Set ONCE ONLY Quick Set Alarm (0=no alarm)
brne	DBP1ab

rcall	QuickSetDisplayAlarm
sbr		r21,1		;set r21,0(1), erase Once Only time, flag


rjmp	DBPend


DBP1ab:	

cpi		r16,10			;is this a 10 hold down count (500)?
breq	DBP1a1			;jump to program command section
rjmp	DBP2


DBP1a1:

pop		r29
pop		r28

rjmp	ProgramCommandStall


DBP2:

cpi		r29,1
brsh	DBP2a
rjmp	DBP2b


DBP2a:

cpi		r28,1
brsh	DBPEnd		;to eliminate invalid numbers


DBP2b:

cpi		r28,1			;this is a relay one toggle
brne	DBP3
rcall	ToggleRelay1
rjmp	DBPEnd


DBP3:

cpi		r28,2			;this is a relay two toggle
brne	DBP4
rcall	ToggleRelay2
rjmp	DBPEnd


DBP4:

cpi		r28,3			;this is a relay three toggle
brne	DBP5
rcall	ToggleRelay3
rjmp	DBPEnd


DBP5:

cpi		r28,4			;this is a relay four toggle
brne	DBP6
rcall	ToggleRelay4
rjmp	DBPEnd


DBP6:					;$0080=Unit No., $0081=command (On, Off, or
						;   Toggle)

sts		$0080,r28			;move unit number to be toggled during next cycle
ldi		r16,4			;put toggle command (bit 2(4)) into next locaton of SRAM
						;bit0=on, bit1=off, bit2=toggle
sts		$0081,r16			;touch plate/time command store area = $0080,$0081


DBPEnd:

pop		r29
pop		r28

sbrs	r19,3			;skip if r19,3(8) is set (master)
rjmp	SlaveLoopA		;jump then use sync up before entering active loop
rjmp	MasterLoopA		;jump after sync up into MasterLoop


;-----------------------------

ButtonStall:	;stalls until button is released

sbic	PIND,2	;skip if bit 2(4) is clear (plate being touched)
ret

rcall	Delay40th
rjmp	ButtonStall


;---------------------------

;CONVERTS number of minutes entered using ButtonPush to SECONDS

ButtonPushX60:		;Entry is made using ButtonPush, then multiplied by
					;  60 to give seconds.  Return is given in r28,29
					
rcall	OneGreenFlash	;signal that one entry is needed

rcall	StallButtonPush

lds		r28,$00EA		;load $00EA, LSD of ButtonPush into r28
lds		r29,$00EB		;load $00EB, MSD of ButtonPush into r29

clr		r30
clr		r31


BPX1:

adiw	r30,60			;add 60 to r30,31, z register pair
sbiw	r28,1			;subtract one from entered count in reg y
brne	BPX1

mov		r28,r30
mov		r29,r31			;put multiplied data into r28,29

ret						;zero flag is set for non-zero result


;-------------------------------------

TimeInterrupt:

rcall	TimeSubroutine

reti


;-------------------------------------

TimeSubroutine: 		;interrupt to update clock counters
						;FORMERLY ClockCntDwnInt (interupt)

push	r16
push	r22
push	r28
push	r29

in		r16,SREG	;save flags
sts		$0060,r16
			
sbr		r19,128		;set r19, bit 7(128) -- one second flag 
					;(set each second)

;  Following section increments or decrements clock by one second WHEN 
;	clock correction interval reaches zero.  Current clock correction 
;	interval is in SRAM $110,111,112,113 (pemanent) and $114,115,116,117 (counter)

;   $118 is storage area for no change (1), subtract second (2), or add second (3)

;	EEPROM location for the 5 register permanent is $10,11,12,13.14 ($14 being the
;     mode of no change, subtract second, add second

lds		r22,$0118	;get SRAM $0118 0=no change, 1=subtract second, 2=add second
tst		r22
breq	CorEnd		;jump out if zero, otherwise decrement counter

lds		r28,$0114
lds		r29,$0115	;load low bytes of counter

sbiw	r28,1		;subtract one from r28,29 word
sts		$0114,r28
sts		$0115,r29	;store low bytes back
brcc	CorEnd

lds		r28,$0116
lds		r29,$0117	;high bytes of counter

sbiw	r28,1		;subtract one from r28,29 (high bytes)	
sts		$0116,r28
sts		$0117,r29	;store high bytes back
brcc	CorEnd

;Store new count data from permanent store area $0110-0113

lds		r22,$0110
sts		$0114,r22

lds		r22,$0111
sts		$0115,r22

lds		r22,$0112
sts		$0116,r22

lds		r22,$0113	;get all permanent counter data to counter - reset count
sts		$0117,r22

lds		r22,$0118	;get code for subtract or add
cpi		r22,1		;ONE equals subtract one second
breq	Cor1

cpi		r22,2
brne	CorEnd

sbr		r19,32		;Set r19,5(32), flag for one second increment
rjmp	CorEnd


Cor1:				;Decrement flag set

sbr		r19,4		;Set r19,2(4) flag for one second increment


CorEnd:


;-------------------------------------

CC1b:

lds		r16,$006B
inc		r16			;step one count of SECONDS counter
sts		$006B,r16

;---------------------------------------

;  At 5 seconds in, check is made for clock correction if flag is set

cpi		r16,5		;IF five seconds in and speed up flag is set, then speed up clock
					;  by one second
brne	CC1bb

sbrs	r19,2		;skip if r19,2(4) bit set - decrement seconds counter
rjmp	CC1b1

cbr		r19,4		;clear r19,2(4) - decrement seconds counter flag
dec		r16			;decrement seconds counter
sts		$006B,r16	;put away altered seconds count
rjmp	CC1bb


CC1b1:

sbrs	r19,5		;skip if r19,5(32) is set - increment seconds counter flag
rjmp	CC1bb

cbr		r19,32		;clear r19,5(32) - increment seconds counter flag
inc		r16			;speed up clock by one second
sts		$006B,r16	;store new seconds count

;----------------------------------------

CC1bb:

cpi		r16,60
breq	CCDI1

rjmp	CCDIExit		;return since no action required


CCDI1:

clr		r16			;zero r16 -- SECONDS counter
sts		$006B,r16

sbr		r19,64		;set R19 bit 6(64) -- the NEW ONE MINUTE flag
					;  Used by Display Clock mode 


CCDI1Cntr:

lds		r16,$0068		;load low minutes byte into R16

inc		r16
sts		$0068,r16

brne	CCDI1a		;jump out if not zero (no overflow)

lds		r16,$0069		;high byte minutes
inc		r16			;increment high byte minutes
sts		$0069,r16


CCDI1a:

lds		r16,$0069		;check high byte if it is 5 (1440=  5 A0)
cpi		r16,5

breq	CCDI1b

rjmp	CCDIalarmExit


CCDI1b:			

lds		r16,$0068
cpi		r16,160		;1440 minutes reached -- it's now midnight

breq	CCDI1c

rjmp	CCDIalarmExit


CCDI1c:			;zero high and low bytes - it is midnight

cbr		r18,96		;clear r18,5(32), r18,6(64) dusk dawn flags

clr		r16
sts		$0068,r16		;store away new start at zero of 1440 count
sts		$0069,r16

lds		r16,$006A		;get day of week and increment
inc		r16
sts		$006A,r16

cpi		r16,8

brne	CCDIalarmExit


CCDI1d:

ldi		r16,1		;load with 1 (Sunday)
sts		$006A,r16


CCDIalarmExit:

;Quick Set Alarm Section - independent from Check All Alarms - added 12/22/2011


DCA1:				;start

lds		r16,$011B	;load 60 minute ON counter (leaves circuit on for one minute)
tst		r16
breq	DCA2

dec		r16
sts		$011B,r16	;store new value into SRAM

brne	DCAend

sbi		PortB,5		;pulse relay #1 off
rcall	Delay40th
cbi		PortB,5
cbr		r17,1		;turn r17,0(1), relay #1 flag off

rjmp	DCAend


DCA2:

lds		r28,$0119
lds		r29,$011A	;load alarm time into r28,29

mov		r16,r28
or		r16,r29		;test for zero

brne	DCA4

rjmp	DCAend


DCA4:

lds		r16,$0068	;get low time byte
cp		r16,r28
brne	DCAend

lds		r16,$0069	;get high time byte
cp		r16,r29
brne	DCAend

;Following checks for erase alarm time flag and if set, clears alarm time in SRAM
;  (Once Only Alarm)

sbrs	r21,0		;skip if r21,0(1) bit is set - Erase Alarm Time flag
rjmp	DCA4a		;This is NOT a Once Only Alarm

cbr		r21,1		;clear r21,0(1) bit - Erase Alarm Time bit

clr		r16			;This is a once only alarm so clear alarm in SRAM

sts		$0119,r16
sts		$011A,r16	;reset alarm time to zero


DCA4a:

sbi		PortB,4
rcall	Delay40th
cbi		PortB,4		;pulse relay #1 on
sbr		r17,1		;turn r17,0(1), Relay #1 flag on

ldi		r16,60		;put 60 minutes on time into 60 minute ON counter
sts		$011B,r16


DCAend:

rcall		CheckAllAlarms


CCDIExit:		;Exit this interrupt routine

lds		r16,$0060		;SREG temporary store
out		SREG,r16	;restore SREG data

pop		r29
pop		r28
pop		r22
pop		r16

ret					;slave unit exit (rcall exit)


;------------------------------------------

CheckAllAlarms:		;check entire list of alarm commands for match

push	r20			;bookmark counter
push	r25			;scratch store
push	r26			;X register LSB
push	r27			;X register MSB
push	r28			;scratch for command, DOW, MSbits
push	r29			;scratch for remote unit number


clr		r27			;zero high byte of X register pair

ldi		r26,$90		;low byte -- start point of alarm memory - x register
lds		r20,$008F		;get present bookmark locaiton for command memory


CAA1:				;check for end of list

cp		r26,r20		;compare r26 against r20
brsh	CAAEnd		;jump out if number same or exceeds bookmark total

ld		r25,x+		;load r25 with LSB of time - post icrement r26,r27
ld		r28,x+		;load r28 with command, DOW, MSbits of time - post increment
ld		r29,x+		;load r29 with remote unit number - post increment

lds		r16,$0068		;load LSB of current time into r16 for comparison
cp		r16,r25
brne	CAA1

mov		r16,r28
andi	r16,7		;isolate MSbits of time MSBits

lds		r25,$0069		;load high time byte (r25 now scratch register)
cp		r16,r25
brne	CAA1

lds		r25,$006A		;load current Day of Week (DOW)
mov		r16,r28		;get data byte with DOW content

lsr		r16
lsr		r16
lsr		r16			;move data into position for check
andi	r16,7		;isolate DOW information 

cpi		r16,0
breq	CAA3

cp		r16,r25
brne	CAA1


CAA3:				;Check now for command to be executed

mov		r25,r28		;get byte with command data
rol		r25
rol		r25
rol		r25			;shifts left through carry flag to position bits (requires three shifts)
andi	r25,3		;isolate two moved MS bits for command (1=on, 2=off, 3=toggle)

cpi		r25,1		;ON command
brne	CAA3a

sts		$0080,r29		;Put unit number into $0080 TP command send
ldi		r16,1		;On bit 1 for ON command
sts		$0081,r16		;store for TP send
rjmp	CAA1


CAA3a:

cpi		r25,2		;OFF command
brne	CAA3b

sts		$0080,r29		;Put remote unit number into $0080 TP command send
ldi		r16,2		;OFF command in bit 1
sts		$0081,r16		;store for TP send
rjmp	CAA1


CAA3b:

cpi		r25,3		;TOGGLE command
brne	CAA1

sts		$0080,r29		;put remote unit number into $0080 TP command send
ldi		r16,4		;TOGGLE command in bit 2
sts		$0081,r16		;store for TP send
rjmp	CAA1


CAAEnd:

pop		r29
pop		r28
pop		r27
pop		r26
pop		r25
pop		r20

ret


;-------------------------------------------------------

ClockSelect:		;Selects how to access Dsiplay Clock

rcall	ShortBeep2

sbrs	r19,3		;skip if r19,3(8) is set - master
rjmp	CSelect1

rcall	DisplayClock	;it is master so display clock normally

rjmp	PCOMend


CSelect1:				;it is slave  

rcall	DisplayClock

rjmp	PCOMend


;--------------------------

DisplayClock:		;displays time
					;DAH count exits this routine

push	r4
push	r5 
push	r6
push	r7
push	r20
push	r28
push	r29


DCP1:

cbr		r19,16		;clear r19,4(16) - OFF quick exit of delays flag
sbi		PortB,1
sbi		PortB,2		;off red and green LEDs

sbis	PIND,2		;skip if button up
rjmp	DCService	;service push button

lds		r20,$006b		;load seconds counter
cpi		r20,0		;0 seconds in
brne	DCP2

ldi		r16,10		;ten wags of R/G LEDS upon entry of "0" clock display time
rcall	KWag		;begin clock display with wag of LEDs
sbis	PIND,2		;skip if switch is up
rjmp	DCService	;service push button

rjmp	DCPtime	


DCP2:

cpi		r20,30		;is it 30 seconds into seconds?
brne	DCP1

ldi		r16,5		;five wags of R/G LEDS upon entry
rcall	KWag		;begin clock display with wag of LEDs

sbis	PIND,2		;skip if switch is up
rjmp	DCService	;service push button

rjmp	DCPtime


DCService:

cbr		r19,16		;clear r19,4(16) Kill delay flag
sbi		PortB,1
sbi		PortB,2		;off red and green LEDs

rcall	ButtonPush
lds		r16,$00EA	;LSB of button push

cpi		r16,5
brlo	DCService1	;Service relays

cpi		r16,10		
brne	DCP30
rjmp	DCEnd		;EXIT routine and return to normal OBH2 operation


DCP30:

cpi		r16,30		;if 3 long touches, then Set the Quick Alarm - Every day
					;  number of quarter hour segments past midnight +1
brne	DCP40

rcall	QuickSetDisplayAlarm	;allows quick alarm entry
rjmp	DCP1		;out - go back to Display Clock


DCP40:				;Once Only Alarm Set

cpi		r16,40		;if 4 long touches, then Once Only Alarm Set
brne	DCP50

rcall	QuickSetDisplayAlarm
sbr		r21,1		;set r21,0(1), Erase Alarm Time flag

rjmp	DCP1


DCP50:				;Five DAH touches will allow setting the clock

cpi		r16,50
breq	DCP50a

ldi		r16,2			;prepare wag routine for two wags of red green LED
rcall	Wag

rjmp	DCP1



DCP50a:

rcall	SetClock	;Allows clock to be set while in this mode

rjmp	DCP1


DCService1:

cpi		r16,1
breq	DCPrelay1
cpi		r16,2
breq	DCPrelay2
cpi		r16,3
breq	DCPrelay3
cpi		r16,4
breq	DCPrelay4

rjmp	DCP1



DCPrelay1:

rcall	ToggleRelay1	;toggle relay1 and jump back into display clock loop
rjmp	DCP1			;loop back


DCPrelay2:

lds		r16,$006F		;get number of active relays
cpi		r16,2
brsh	DCPrelay2a
rjmp	DCP1			;loop back


DCPrelay2a:

rcall	ToggleRelay2
rjmp	DCP1			;loop back


DCPrelay3:

lds		r16,$006F		;get number of active relays
cpi		r16,3
brsh	DCPrelay3a
rjmp	DCP1			;loop back


DCPrelay3a:

rcall	ToggleRelay3
rjmp	DCP1			;loop back


DCPrelay4:

lds		r16,$006F		;get number of active relays
cpi		r16,4
brsh	DCPrelay4a
rjmp	DCP1			;loop back


DCPrelay4a:

rcall	ToggleRelay4
rjmp	DCP1			;loop back


DCPtime:				;START CLOCK DISPLAY

rcall	KillDelay2sec

clr		r6		;zero hours count
clr		r7		;zero MSD of minutes count

lds		r16,$006A		;load day of week into r16


DCdow:			;green-red-green flash for each day-of-week

cbi		PortB,2		;on green led
rcall	KillDelay40th
sbi		PortB,2		;off green led
cbi		PortB,1		;on red led
rcall	KillDelay40th	
sbi		PortB,1		;off red led
cbi		PortB,2		;on green led
rcall	KillDelay40th	
sbi		PortB,2		;off green led
rcall	KillDelay500ms	;pause between flash

dec		r16
brne	DCdow
rcall	KillDelay500ms	;pause after flashing out series

;Finished flashing out day of week

cli					;OFF interrupts for time data load

lds		r28,$0068		;load low byte into r28
lds		r29,$0069		;load high byte into r29

sei					;turn interrupts back ON

tst		r29		;test high byte for zero
brne	DC3

cpi		r28,60		;check to see if less than 60 (12:00 am to 12:59 am)
brlo	DC2
rjmp	DC3


DC2:

ldi		r16,12		;flash out 12 a.m.

rcall	VarClockFlash	;flash out time of 12 a.m.

rjmp	DC7


DC3:			;flash out other times

mov		r4,r28		;save low byte in r4
mov		r5,r29		;save high byte in r5

inc		r6			;increment hours count
sbiw	r28,60		;subtract 60 from the r28,29 pair

brcs	DC3a		;hours now almost decoded

rjmp	DC3		;repeat loop again


DC3a:

dec		r6		;restore old value of hours
mov		r28,r4		;restore low byte in r28
			;at this point, minutes remainder is in low byte
			; (r29 will be zero)
			

DC10:

mov		r16,r6		;put hours into r16 for check
cpi		r16,13
brsh	DC9
rjmp	DC8


DC9:

subi	r16,12


DC8:			;flash out time

rcall	VarClockFlash


DC7:			;decode minutes with MSD double flashed

mov		r16,r28
inc		r7		;use r7 for MSD count
subi	r28,10	;get remaining minutes
brcs	DC7a

rjmp	DC7		;loop back for another subtraction


DC7a:

dec		r7		;restore correct value for MSD
mov		r28,r16		;restore correct remainder (LSDs)

mov		r16,r7		;put in MSD for flashout
sbr		r19,1		;set r19,0(1) for double flash
rcall	VarClockFlash	;flash out MSD
cbr		r19,1		;clear r19,0(1) double flash flag

mov		r16,r28
rcall	VarClockFlash	;flash out LSDs

rjmp	DCP1


;-----

DCEnd:

rcall	ShortBeep2

pop		r29
pop		r28
pop		r20
pop		r7
pop		r6
pop		r5
pop		r4

ret					;jump out


;---------------------------

VarClockFlash:		;flashes red or green LED (G=AM R=PM)
					;R16 contains number of flashes
					;INTENDED USE is only for DisplayClock (Kill Delay flags used)

sbrc	r19,4		;skip if r19,4(16) is clear (Kill Delay flag)
ret

push	r17

rcall	KillDelay500ms	;delay before flashes start
rcall	KillDelay500ms

lds		r17,$0069		;check if AM or PM time (less than 720 = AM)
cpi		r17,2		;go to AM if high byte is <2 
brlo	VCF1		;jump to AM routine
cpi		r17,2
breq	VCF3
rjmp	VCF2		;jump to PM routine


VCF3:

lds		r17,$0068		;compare low byte for less than 208 - 12:00 noon
cpi		r17,208		
brlo	VCF1		;jump to AM routine
rjmp	VCF2		;jump to PM routine


VCF1:			; AM routine

tst		r16
breq	VCF2a		;clear double flash flag and return

cbi		PortB,2		;turn on GREEN   (AM)
rcall	KillDelay100ms	;on time
sbi		PortB,2		;off Green LED

sbrc	r19,0		;skip if r19,0(1) is clear - no double flash
rjmp	VCF1ab		;go for double flash


VCF1ac:

rcall	KillDelay333ms		;off time
dec		r16
rjmp	VCF1


VCF1ab:			;gives double flash

rcall	KillDelay100ms	;small pause
cbi		PortB,2		;on green LED
rcall	KillDelay40th	;on time
sbi		PortB,2		;off Green LED
rjmp	VCF1ac


VCF2:			; PM routine

tst		r16
breq	VCF2a		;clear double flash flag and return

cbi		PortB,1		;turn on RED LED
rcall	KillDelay100ms	;on time delay
sbi		PortB,1		;turn off RED LED
sbrc	r19,0		;skip if r19,0(1) is clear - no double flash
rjmp	VCF2ab


VCF2ac:

rcall	KillDelay333ms		;off time delay
dec		r16
rjmp	VCF2


VCF2ab:			;give double red flash

rcall	KillDelay100ms	;short off delay
cbi		PortB,1		;on Red LED
rcall	KillDelay40th	;on time delay for second flash
sbi		PortB,1		;off Red LED
rjmp	VCF2ac


VCF2a:

sbi		PortB,1
sbi		PortB,2		;off red and green LEDs

pop		r17
cbr		r19,1		;clear R19,0(1) - double flash flag
ret


;-------------------------------------------

SetClock:	;sets clock for day of week, AM/PM, hour, minutes
			;  24 hour time entry required
			;0 = midnight (1440 minute ounter)

;SPECIAL NOTE:  -- Unit Number, Hours, and Minutes are 
;  done by dit (1), dah(10), and hold down (50) for numbers to be entered.
;  a RED flash precedes all entries that use the dit/dah/hold down button push.
;  PLUS ONE entry system --- zero = one entry,  240=241 entry

push	r1		;minutes accumulator
push	r3		;day of week accumulator
push	r30
push	r31		;accumulator for minutes -- Z register pair

ldi		r16,4		;set up for FOUR green flashes
rcall	FlashGLED	;indicate that four entries are needed

rcall	StallButtonPush	
lds		r16,$00EA		;put data into r3 -- day of week (2=Sunday, 8=Saturday)
dec		r16			;subtract one for proper entry input
andi	r16,7		;trim off any excess minutes
mov		r3,r16		;save in r3 

rcall	OneGreenFlash

rcall	StallButtonPush	;enter hours -- 24 hour format
lds		r16,$00EA
dec		r16			;one less than entry so ZERO entry is made possible
					;  1 = 0,  2 = 1,  3 = 2,  etc.
clr		r30
clr		r31			;zero register pair


SC1a:

tst		r16
breq	SC1b

adiw	r30,60		;add 60 to change hours into minutes
dec		r16
brne	SC1a


SC1b:

rcall	OneGreenFlash	;signal another entry needed

rcall	StallButtonPush	;enter minutes
lds		r16,$00EA			;put Button Push value into r16
dec		r16				;one less to make for zero entry
						;  1 = 0,  2 = 1,  3 = 2,  etc.
add		r30,r16			;add r16 to r30
brcc	SC1d

inc		r31				;add one to r31 (carry)


SC1d:

;minute total now complete

rcall	OneGreenFlash	;signal another entry needed (final entry)



SC1e:

rcall	Delay40th
sbic	PIND,2			;skip if button is pushed
						;START clock upon last button push
rjmp	SC1e


SetEnd:

rcall	ButtonStall		;stall until button is released
						;  clock is set upon release of button

cli						;turn off global interrupts

sts		$006A,r3		;store day-of-week
sts		$0068,r30		;store low byte in $0068
sts		$0069,r31		;store high byte in $0069
clr		r3				;zero seconds counter
sts		$006B,r3		;store zero seconds

sei						;ON interrupts


SetEnd2:

pop		r31
pop		r30
pop		r3
pop		r1

rcall	ShortBeep2

ret

;--------------------------------------

EnterAlarmTimeData:		;This entry creates new book mark at $0090 in $008F

rcall	ClearAllAlarms	


EnterAlarmTimeData2:	;This entry retains old book mark

			;manually enters time commands into command store
			;FIVE entries required - hold down is used for entry 
			;     except Zero which is DIT (short touch)
			; 1) 1 = EXIT, 2 = ON, 3 = OFF,  4 = TOGGLE
			; 2) 1 = every day, 2 (Sun) - 8 (Sat) = Sunday through Saturday
			; 3) Remote Unit Number to 255
			; 4) Hours (24 hour format) 
			; 5) Minutes

			;Byte1= LSD, Byte2=command(2),DOW(3),MSB alarm time.  Byte3= unit number
						
						
push	r0
push	r20
push	r25		;scratch
push	r26		;push X register LSB (Book Mark counter)
push	r27		;push X register MSB
push	r30		;push Z register LSB
push	r31		;push Z register MSB (total time count)

clr		r27		;zero MSB of "X" register pair


EATD1:

lds		r26,$008F		;get bookmark for present location of memory pointer
					; put in r26 (scratch), register
					; SRAM memory for alarm times = $0090 - $00D1
					; byte 1 = LSD of time, byte 2 = bits 0-3 MSD of time
					; byte 2 = bit 4-6 = Day of Week
					; byte 2 = bit 7-8 = Command 1=toggle,  2=on,  3=off
							
cpi		r26,$D2		;check for end of memory storage
brsh	EATD2		;exit if count is same or higher
rjmp	EATD3		;entry okay, ... continue


EATD2:			;give long RED flash showing memory is full ERROR, then EXIT

cbi		PORTB,1		;on LED
rcall	Delay2310	
sbi		PORTB,1		;off LED

rjmp	EATDEnd


EATD3:

ldi		r16,5		;set up for FIVE green flashes
rcall	FlashGLED	;indicate that five entries are needed


;Enter command 1=EXIT, 2=off,  3=on,  4=toggle
rcall	StallButtonPush	;Enter 1 to exit, 2 for ON, 3 for OFF, 4 for TOGGLE
						;   hold down count entry

lds		r20,$00EA		;get hold down value from ButtonPush LSB total count
cpi		r20,1
brne	EATD3a
rjmp	EATDEnd		;jump out if one is entered


EATD3a:

dec		r20			;decrease count. 1=ON, 2=OFF, 3=TOGGLE

lsl		r20
lsl		r20
lsl		r20
lsl		r20
lsl		r20
lsl		r20			;command data now positioned in bits 6 and 7

mov		r25,r20		;store in r25, three MSbits


;Enter DOW command:    1=every day, 2=sunday ....... 8=Saturday
EATD5:

rcall	OneGreenFlash	;indicate single entry is needed 

rcall	StallButtonPush	;Enter 1=every day, 2 (Mon) - 8 (Sat) = Sunday through Saturday
lds		r20,$00EA			;load total count from EA into r20
dec		r20				;decrement to fit count in 3 bits

lsl		r20
lsl		r20
lsl		r20			;store in bits 3,4,5
andi	r20,56		;trim byte in event higher number than 7 was accidentally entered


or		r25,r20		;or the DOW data with command data
					;store to this pointcontains DOW and command data in r25
					; (bits 3,4 5 = DOW;  bits 6,7 = command)

;Enter Unit number
rcall	OneGreenFlash	;indicates single entry for unit number

rcall	StallButtonPush	;Enter Unit Number
lds		r0,$00EA			;load unit number into r0 from $00EA


;Enter Hours in 24 hour format
rcall	OneGreenFlash	;single entry for hours in 24 hour format

clr		r30
clr		r31			;zero Z registers for following process

rcall	StallButtonPush	;Enter dit/dah/hold down count entry for HOURS (24 hr format)
lds		r20,$00EA		;load total count from $00EA into r20
dec		r20			;decrement to make room for ZERO entry at 1
					;  all entries are hour plus ONE!
andi	r20,31		;trim byte in case of incorrect entry (less that 31)


EATD5a:

tst		r20		
breq	EATD5b

adiw	r30,60		;add 60 minutes for number of times in r1
dec		r20	
brne	EATD5a


EATD5b:

rcall	OneGreenFlash	;single entry for total minutes

rcall	StallButtonPush	;Enter number of minutes
lds		r20,$00EA	;load total count in $00EA
dec		r20		;ALL minutes are entered as minutes plus ONE!
				;  thus, ZERO minute = 1
andi	r20,63	;limit maximum entry amount to 63
	
add		r30,r20	;add r20 to r30 to word r30,31 
brcs	EATD5d
rjmp	EAT5e


EATD5d:

inc		r31		;add in carried digit to MSD of minutes count
andi	r31,7	;trim byte in case of entry error - limit to 3 bits


EAT5e:
or		r25,r31	;OR MSbits of minutes. Byte now complete for store


EATD5c:			;completed data ready for store

st		x+,r30		;store LSD of minutes in alarm memory - post incr
st		x+,r25		;store MSD (command, day-of-week, MSbits of minutes - post incr
st		x+,r0		;store unit number - post incr

sts		$008F,r26		;store x register value in r26, -- new bookmark

rjmp	EATD1		;go back for new entry


EATDEnd:

pop		r31
pop		r30
pop		r27
pop		r26
pop		r25
pop		r20
pop		r0

rcall	ShortBeep2

ret


;------------------------------------------


AtomicClockReset:	;resets clock to signal from Atomic Clock at 4:00 a.m.

sbic	PIND,4		;skip if 4:00 a.m. PIND,4(16) reset signal is present
ret					;signal not present

push	r16

lds		r16,$00B2		;atomic clock DELAY countdown
tst		r16
brne	ACREnd

ldi		r16,3		;Set a delay so repeat cycles are not done
sts		$00B2,r16		;set three minutes (3) delay, to prevent retrigger.

			;This section resets day-of-week to 2 (Monday)
			
bclr	7		;disable global interrupts

clr		r15		;zero seconds counter

ldi		r16,240
sts		$0068,r16	
clr		r16
sts		$0069,r16		;put 4 into hours (4 a.m.)

bset	7		;restore global interrupts

lds		r16,$00B3		;load Atomic Clock DaysMissedDaysCounter count
cpi		r16,3	
brlo	ACR1		;branch if lower than 3 (3 = Monday)

ldi		r16,2		;put 2 (Monday) into Day of Week Counter, R25
sts		$006A,r16		; -- a weekend of two days without alarm has passed


ACR1:

clr		r16		;zero Atomic DaysMissedCounter	
sts		$00B3,r16		; and store away


ACREnd:

pop		r16

ret


;-----------------------------------------

Mult100:		;Multiplies value in R20 by 100 and returns value in R20

push	r1
clr		r1	;zero r1 for upcoming data

tst		r20
breq	Mult100b

ldi		r16,100	;the multiplication value


Mult100a:

add		r1,r16
dec		r20
brne	Mult100a


Mult100b:

mov		r20,r1	;for return of value
pop		r1
ret


;--------------------------------------

Mult10:		; multiplies value in R20 by 10 returning value in R20

push 	r1
clr		r1		;zero R1

tst		R20		;test long count value
breq	Mult10b		;value is zero

ldi		r16,10		;put ten into r16

Mult10a:

add		r1,r16		;add ten for each value in R1
dec		r20
brne	Mult10a

Mult10b:

mov		r20,r1		;move r1 value into r20
pop		r1
ret


;-------------------------------------

SetClockOneSecondCorrection:	;Sets one second speedup/slowdown at entered interval

;	FIVE entries required.. Wait for green light flash before making next entry.

;	First entry is 16,777,216 units (0-255).  Seond is 65,536 units (0-255),  Third is 256 units.  Fourth and final
;	  is single units (0-255).  Fifth and last is whether you wish this to be a
;	  (0) no change, (1) subtract second,  (2) add second.

;	Keep in mind that ALL entries use PLUS ONE system.  That is if you wish to enter
;     a zero, you push the button once.  A four is entered by pushing the button
;     five times,  223 is entered by keying in 224, etc.


push	r28
push	r29
push	r30
push	r31

ldi		R16,5		;prepare for five flashes - five entries
rcall	FlashGLED

rcall	StallButtonPush	;Enter 16,777,216 units entry (1-256)
lds		r16,$0100		;Get ONE LESS entry ($0100) from Button Push
mov		r31,r16		;place in maximum count register, r31


rcall	OneGreenFlash	;Alert that it is ready for another entry

rcall	StallButtonPush	;Enter 65536 units entry (1-256)
lds		r16,$0100		;Get ONE LESS entry ($0100) from Button Push
mov		r30,r16		;place in next lowest count register, r30


rcall	OneGreenFlash	;Alert that it is ready for another entry

rcall	StallButtonPush	;Enter 256 units entry (1-256)
lds		r16,$0100		;Get ONE LESS entry ($0100) from Button Push
mov		r29,r16		;place in next lowest count register, r29


rcall	OneGreenFlash	;Alert that it is ready for another entry

rcall	StallButtonPush	;Enter single units entry (1-256)
lds		r16,$0100		;Get ONE LESS entry ($0100) from Button Push
mov		r28,r16			;place in next lowest count register, r28


rcall	OneGreenFlash	;Alert that it is ready for another entry

rcall	StallButtonPush	;Enter 0=No Correction, 1=One second slow down, 2=one second speed up
						; (using plus one entry system -- 1 push = zero entry)
lds		r16,$0100		;Get ONE LESS entry ($0100) from Button Push

sts		$0118,r16	;put away mode 0= no change, 1= subtract one sec, 2= add one sec.
					;  SRAM location = $0118


COSC1:				;All entries now in

cli					;OFF interrupts

sts		$0113,r31
sts		$0112,r30		;save count into permanent SRAM
sts		$0111,r29
sts		$0110,r28

sts		$0117,r31
sts		$0116,r30		;save count into interval counter
sts		$0115,r29
sts		$0114,r28

sei					;ON interrupts

;Following saves permanent interval in EEPROM

lds		r16,$0110		;get lowest byte of data for storage

ldi		r30,$10		;Set up start of EEPROM store location in r30,31
clr		r31

rcall	writeEEPROM	;put low byte of interval into EEPROM - $10
inc		r30			;increment data location in EEPROM

lds		r16,$0111		;

rcall	writeEEPROM	;location = $11
inc		r30

lds		r16,$0112	;get next highest data

rcall	writeEEPROM ;location $12
inc		r30

lds		r16,$0113	;get lowest data byte
rcall	writeEEPROM	;location $13
inc		r30

lds		r16,$0118	;get mode  0=no change, 1=subtract second,  2= add second
rcall	writeEEPROM	;location $14

pop		r31
pop		r30
pop		r29
pop		r28

rcall	ShortBeep2

ret


;--------------------------


ToggleRelay1:		;toggles relay circuit #1

lds		r16,$011B	;load display clock 60 minute ON count
tst		r16
breq	TR1a

clr		r16
sts		$011B,r16	;Kill count in $011B


TR1a:

sbrc	R17,0		;skip if R17,0(1), Relay Flag 1, is clear (relay #1 off)
rjmp	TAC1a

sbi		PortB,4
rcall	Delay40th
cbi		PortB,4		;pulse relay #1 on
sbr		r17,1		;turn r17,0(1), Relay #1 flag on


TRAutoOff1:			;This section loads seconds count from $0062,$0063
					;   into $007A, $007B to activate auto off for relay 1.
					; Activated only if permanent count in Auto Off is not Zero

lds		r16,$0062	;LSB
sts		$007A,r16
lds		r16,$0063	;MSB
sts		$007B,r16	;put perm store value into temp count down for Auto Relay1 Off

ret


TAC1a:

sbi		PortB,5		;pulse relay #1 off
rcall	Delay40th
cbi		PortB,5
cbr		r17,1		;turn r17,0(1), relay #1 flag off


TRAutoOffClear1:

clr		r16
sts		$007A,r16
sts		$007B,r16		;clear auto off delay count for relay 1

ret


;----------------------------------


ToggleRelay2:		;toggles relay circuit #2

sbrc	R17,1		;skip if R17,1(2), Relay Flag 2, is clear (relay #2 off)
rjmp	TAC2a

sbi		PortC,0		;set bit Port C0(1) on -- relay#2
rcall	Delay40th
cbi		PortC,0		;ON bit
sbr		r17,2		;turn r17,1(2), Relay #2 flag on

ret


TAC2a:

sbi		PortC,1		;pulse relay #2 off - Port C0(1) 
rcall	Delay40th
cbi		PortC,1
cbr		r17,2		;turn r17,1(2), relay #2 flag off

ret


;----------------------------------

ToggleRelay3:		;toggles relay circuit #3

sbrc	R17,2		;skip if R17,2(4), Relay Flag 3, is clear (relay #3 off)
rjmp	TAC3a

sbi		PortC,2		;pulse relay #3 on
rcall	Delay40th
cbi		PortC,2
sbr		r17,4		;turn r17,2(4), Relay #3 flag on

ret


TAC3a:

sbi		PortC,3		;pulse relay #3 off
rcall	Delay40th
cbi		PortC,3
cbr		r17,4		;turn r17,2(4), relay #3 flag off

ret


;--------------------------

ToggleRelay4:		;toggles relay circuit #4

sbrc	R17,3		;skip if R17,3(8), Relay Flag 4, is clear (relay #4 off)
rjmp	TAC4a

sbi		PortC,4		;pulse relay #4 on
rcall	Delay40th
cbi		PortC,4
sbr		r17,8		;turn r17,2(4), Relay #4 flag on

ret


TAC4a:

sbi		PortC,5		;pulse relay #4 off
rcall	Delay40th
cbi		PortC,5
cbr		r17,8		;turn r17,3(8), relay #4 flag off

ret


;--------------------------------


OnRelay1:			;On relay #1

sbi		PortB,4
rcall	Delay40th
cbi		PortB,4		;Pulse relay #1 ON
sbr		R17,1		;set R17,0(1) to ON

clr		r16
sts		$007A,r16
sts		$007B,r16	;zero any pending auto delay off

ret


;--------------------------

OnRelay2:

sbi		PortC,0
rcall	Delay40th
cbi		PortC,0		;Pulse relay #2 ON
sbr		R17,2		;set r17,1(2) to ON

ret


;-----------------------------

OnRelay3:

sbi		PortC,2
rcall	Delay40th
cbi		PortC,2		;Pulse relay #3 ON
sbr		R17,4		;set r17,2(4) to ON

ret


;--------------------------

OnRelay4:

sbi		PortC,4		;pulse relay #4 on
rcall	Delay40th
cbi		PortC,4
sbr		r17,8		;turn r17,2(4), Relay #4 flag on

ret

;---------------------------


OffRelay1:

sbi		PortB,5
rcall	Delay40th
cbi		PortB,5		;Pulse relay #1 OFF
cbr		R17,1		;clear r17,0(1) to OFF

ret


;-------------------------

OffRelay2:

sbi		PortC,1
rcall	Delay40th
cbi		PortC,1		;Pulse relay #2 OFF
cbr		R17,2		;clear r17,1(2) to OFF

ret


;---------------------------

OffRelay3:

sbi		PortC,3
rcall	Delay40th
cbi		PortC,3		;Pulse relay #3 OFF
cbr		R17,4		;clear r17,2(4) to OFF

ret


;----------------------------------------------

OffRelay4:

sbi		PortC,5		;pulse relay #4 off
rcall	Delay40th
cbi		PortC,5
cbr		r17,8		;turn r17,3(8), relay #4 flag off

ret

;------------------------------------


;  DELAY ROUTINES

;---------------------


;Delay Ladder

Delay100:	nop
Delay99:	nop
Delay98:	nop
Delay97:	nop
Delay96:	nop
Delay95:	nop
Delay94:	nop
Delay93:	nop
Delay92:	nop
Delay91:	nop
Delay90:	nop
Delay89:	nop
Delay88:	nop
Delay87:	nop
Delay86:	nop
Delay85:	nop
Delay84:	nop
Delay83:	nop
Delay82:	nop
Delay81:	nop
Delay80:	nop
Delay79:	nop
Delay78:	nop
Delay77:	nop
Delay76:	nop
Delay75:	nop
Delay74:	nop
Delay73:	nop
Delay72:	nop
Delay71:	nop
Delay70:	nop
Delay69:	nop
Delay68:	nop
Delay67:	nop
Delay66:	nop
Delay65:	nop
Delay64:	nop
Delay63:	nop
Delay62:	nop
Delay61:	nop
Delay60:	nop
Delay59:	nop
Delay58:	nop
Delay57:	nop
Delay56:	nop
Delay55:	nop
Delay54:	nop
Delay53:	nop
Delay52:	nop
Delay51:	nop
Delay50:	nop
Delay49:	nop
Delay48:	nop
Delay47:	nop
Delay46:	nop
Delay45:	nop
Delay44:	nop
Delay43:	nop
Delay42:	nop
Delay41:	nop
Delay40:	nop
Delay39:	nop
Delay38:	nop
Delay37:	nop
Delay36:	nop
Delay35:	nop
Delay34:	nop
Delay33:	nop
Delay32:	nop
Delay31:	nop
Delay30:	nop
Delay29:	nop
Delay28:	nop
Delay27:	nop
Delay26:	nop
Delay25:	nop
Delay24:	nop
Delay23:	nop
Delay22:	nop
Delay21:	nop
Delay20:	nop
Delay19:	nop
Delay18:	nop
Delay17:	nop
Delay16:	nop
Delay15:	nop
Delay14:	nop
Delay13:	nop
Delay12:	nop
Delay11:	nop
Delay10:	nop
Delay9:		nop
Delay8:		nop

Delay7:		ret


;-------

DelayPoint1:		;.1 second delay

push 	r1
push 	r16

clr		r1
ldi		r16,233


DP1a:

nop
nop
nop
dec		r1
brne	DP1a
dec		r16	
brne	DP1a

pop		r16
pop		r1
	
ret

;--------------------


Delay2310:

push	r31
ldi		r31,24
rjmp	DelayStart




Delay495:

push	r31
ldi		r31,6
rjmp	DelayStart


Delay330:

push	r31
ldi		r31,2
rjmp	DelayStart


Delay165:

push	r31
ldi		r31,1


DelayStart:

push	r0
push	r1

clr		r0
clr		r1		;zero registers

DelayA:

dec		R0
brne	DelayA

dec		r1
brne	DelayA

dec		r31
brne	DelayA

pop		r1
pop		r0
pop		r31

ret


;------------------------

Delay80th:			;1/80th second delay

push	r16
ldi		r16,220


D80a:

rcall	Delay100
rcall	Delay100

dec		r16
brne	D80a

rcall	Delay73
pop		r16
ret


;--------------------------------


Delay40th:			;1/40th second delay (actual=40.30746796 per second)
					;  (24.80929839-3 seconds)(slightly less than 1/40 sec)
rcall	Delay80th
rcall	Delay80th

ret


;-------------------------

Delay1sec:

push	r16
ldi		r16,80


Delay1sec1:

rcall	Delay80th	;24.80929839-3/sec
dec		r16
brne	Delay1sec1


pop		r16

ret


;------------

Delay2sec:		;two second delay

push	r16
ldi		r16,160

rjmp	Delay1sec1


;------------------


Delay333ms:

push	r16
ldi		r16,27

rjmp	Delay1sec1


;--------------


Delay500ms:		;500 millisecond delay

push	r16
ldi		r16,40

rjmp	Delay1sec1


;--------------

Delay250ms:		;250 millisecond delay

push	r16
ldi		r16,20

rjmp	Delay1sec1


;-----------------

Delay150ms:

push	r16
ldi		r16,12

rjmp	Delay1sec1


;-------------------

Delay100ms:

push	r16
ldi		r16,8

rjmp	Delay1sec1


;---------------------

Delay50ms:

push	r16
ldi		r16,4

rjmp	Delay1sec1


;---------------------


Delay10U:				;10 microsecond delay (36 clock cycles)

push	r16
ldi		r16,8


D10a:

dec 	r16
brne	D10a

nop	
pop		r16

ret


;----------------------

;  KILL DELAY ROUTINES - zip through delay when button is pushed during a 1/40th delay

;-----------------------


KillDelay40th:		;cancels delay with r19,4(16) flag is set
					;sets flag when touch plate is touched

push	r0
push	r20

clr		r0
ldi		r20,49


KD40a:

sbrc	r19,4		;skip if r19,4(16) is clear (Kill Delay Flag)
rjmp	KD40End

sbis	PIND,2		;skip if TP not touched
rjmp	KD40b

dec		r0
brne	KD40a

dec		r20
brne	KD40a


KD40End:

pop		r20
pop		r0

ret


KD40b:

sbr		r19,16		;set r19,4(16) (Kill Delay flag)
rjmp	KD40End


;-------------------------------

KillDelay50ms:

sbrc	r19,4

ret

push	r16
ldi		r16,2


KD50a:

rcall	KillDelay40th
dec		r16
brne	KD50a

pop		r16

ret


;--------------------------------

KillDelay100ms:		

sbrc	r19,4

ret

push	r16
ldi		r16,4


KD100a:

rcall	KillDelay40th
dec		r16
brne	KD100a

pop		r16

ret


;-------------------------------


KillDelay150ms:

sbrc	r19,4		;skip if r19,4(16) is clear (Kill delay flag)

ret

push	r16
ldi		r16,6


KD150a:

rcall	KillDelay40th
dec		r16
brne	KD150a

pop		r16

ret


;-----------------------------------------

KillDelay250ms:		;1/4 second delay

sbrc	r19,4		;skip if r19,4(16) is clear (Kill Delay Flag)

ret

push	r16

ldi		r16,10


KD250a:

rcall	KillDelay40th
dec		r16
brne	KD250a

pop		r16

ret


;--------------------------------

KillDelay333ms:		;333ms delay can be cancelled by setting of r19,4(16) kill delay flag

sbrc	r19,4		;skip if r19,4(16) is clear (Kill delay flag)

ret

push	r16
ldi		r16,13


KD333a:

rcall	KillDelay40th
dec		r16
brne	KD333a

pop		r16

ret


;---------------------------------

KillDelay500ms:		;1/2 second delay

sbrc	r19,4		;skip if r19,4(16) is clear (Kill Delay Flag)

ret

push	r16
ldi		r16,20


KD500a:

rcall	KillDelay40th
dec		r16
brne	KD500a

pop		r16

ret


;---------------------------------

KillDelay2sec:		;2 second delay

sbrc	r19,4		;skip if r19,4(16) is clear (Kill delay flag)

ret

push	r16
ldi		r16,81


KD2a:

rcall	KillDelay40th
dec		r16
brne	KD2a

pop		r16

ret


;---------------------------------

KillDelay4sec:

sbrc	r19,4		;skip if r19,4(16) is clear (Kill delay flag)

ret

push	r16
ldi		r16,162


KD4a:

rcall	KillDelay40th
dec		r16
brne	KD4a

pop		r16

ret


;----------------------------------

Pulse60Usec:	;10 microsecond pulse (229 clock cycles)

sbi		PortB,0		;On pulse, PortB,0(1)
push	r16
ldi		r16,72


PUsec1:

dec		r16
brne	PUsec1

cbi		PortB,0		;Off pulse,  PortB,0(1)
pop		r16

ret


;--------------------------------------


FlashGLED:		;Flash Green LED no. of times in R16

push	r0
mov		r0,r16

tst		r16
breq	FlashGLEDend


FlashGLEDa:

cbi		PORTB,2			;Turn on GREEN LED
rcall	Delay150ms
sbi		PORTB,2			;Off GREEN LED
rcall	Delay250ms
dec		r0
brne	FlashGLEDa


FlashGLEDend:

pop		r0

ret


;------------------------------------

Wag:				;r16 contains number of wags

cbi		PortB,2		;on green led (to give one wag)
rcall	Delay50ms
sbi		PortB,2		;off green led
cbi		PortB,1		;on red led
rcall	Delay50ms
sbi		PortB,1		;off red led

dec		r16
brne	Wag

ret

;------------------------------------------

KWag:				;r16 contains number of wags - no display when T.P. is touched

sbrc	r19,4		;skip if r19,4(16) is clear - kill delay flag
rjmp	KWag1

cbi		PortB,2		;on green led (to give one wag)
rcall	KillDelay50ms
sbi		PortB,2		;off green led
cbi		PortB,1		;on red led
rcall	KillDelay50ms
sbi		PortB,1		;off red led

dec		r16
brne	KWag


KWag1:

ret


;------------------------------------------

DoubleGreenFlash:

cbi		PortB,2		;on green LED
rcall	Delay100ms
sbi		PortB,2		;off green LED
rcall	Delay150ms


OneGreenFlash:

cbi		PortB,2		;on green LED
rcall	Delay100ms	
sbi		PortB,2		;off green LED

ret

;------------------------------------


TwoGreenFlash:

ldi		R16,2		;prepare for two flashes
rcall	FlashGLED

ret


;-------------------------------------

LongRedFlash:	;for error marking

cbi		PortB,2		;on green LED
rcall	Delay1sec
sbi		PortB,2		;off green LED

ret


;------------------------------------

SyncUp:			;Waits for .311 seconds of no pulse (enough for 255 unit scans)
				;   before returning.  Also turns off interrupts if in slave mode.

push	r0


SU1:

clr		r16
clr		r0

sbis	PIND,0	;input control line pulse, skip if no pulse
rjmp	SU1


SU2:

sbis	PIND,0	;input control line pulse, skip if no pulse
rjmp	SU1

rcall	Delay12

dec		r0
brne	SU2

dec		r16
brne	SU2

pop		r0

ret


;-----------------------------
SinglePulse:		;394 clock cycles

push	r16
cbr		r18,2		;clear pulse detected flag r18,1(2)
sbrc	r18,0		;skip if r18,0(1) is clear (receive pulse)
rjmp	SP1			;(send 60 us pulse)


SP2:				;(receive pulse)

ldi		r16,75



SP4:

sbis	PIND,0		;skip if PIND,0 is set (not active)
sbr		r18,2		;pulse detected - set r18,1(2)


SP3:

dec		r16
brne	SP4
push	r16
pop		r16
nop					;(5 clock delays)		


SPEnd:

pop		r16

ret


SP1:  	;(send)

rcall	Pulse60usec		;(229)
rcall	Delay100
rcall	Delay48
rjmp	SPEnd			;388 to this point


;--------------------------------------

WriteEEPROM:		;write EEPROM byte - r30,r31 (z register) has EEPROM address,
					; r16 has data, 

push	r17
                                                

WriteE1:

sbic	EECR,EEWE	;wait until ready before continuing
rjmp	WriteE1

in		r17,SREG	;Store SREG to avoid interrupt conflict
cli					;Off interrupts so write not corrupted if interrupt occurs

out		EEARH,r31		;set high byte of address (0)
out		EEARL,r30		;set up low byte address in address register
out		EEDR,r16		;put contents of r16 into EEPROM data location

sbi		EECR,EEMWE	;write logical one to EEMWE

sbi		EECR,EEWE	;start eeprom write by setting EEWE

out		SREG,r17

sbrc	r19,3		;skip if r19,3(8) is clear (slave)
sei					;On interrupts if Master

pop		r17

ret	


; ----------------------------------------

FetchEEPROM:		;r30,31 (z register) has address
;					;r16 will contain data from EEPROM

cli					;off global interrupts

FEE1:

sbic	EECR,EEWE	;Wait for completion of previous write
rjmp	FEE1

out		EEARH,r31	;set high bit address
out		EEARL,r30	;set up low address
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r16,EEDR	;read data from data register

sbrc	r19,3		;skip if r19,3(8) is clear (slave)
sei					;On interrupts if Master

ret					

;-------------------------------------------------

; 23 clock cycles for QuickInputsCheck - Used in programming Unit 0 command

QuickInputscheck:	;immediately checks for inputs (28 clock cycles)
; slot1= 4 am correction r29,0(1)  slot2= Prox. Detector r29,1(2)
; slot3= Dusk/Dawn r29,2(4)    slot4= Burglar Alarm r29,3(8)
; contents of r29 stored in E6 - r29 is a global flags register.


; Slot 1 = 4 a.m.                r29,0(1)  PIND,4
; Slot 2 = proximity detector    r29,1(2)  PIND,5
; Slot 3 = dusk/dawn			 r29,2(4)  PIND,1
; Slot 4 = burglar alarm         r29,3(8)  PIND,3

; Slot 5 = (Sunday DOW correction pulse send)


sbis	PIND,4		;4am positive signal
rjmp	QIC1

sbr		r29,1		;set r29,0(1), 4am bit high


QIC1:

sbis	PIND,5		;input proximity detector (positive signal)
rjmp	QIC2

sbr		r29,2		;set r29,1(2) high


QIC2:

sbis	PIND,1		;skip if dusk-dawn, PIND,1 is set (night)
rjmp	QIC2a

sbr		r29,4		;set r29,2(4) night=1
rjmp	QIC3


QIC2a:

cbr		r29,4		;clear r29,2(4) day=0
nop


QIC3:

sbis	PIND,3		;Burglar alarm positive signal detection
rjmp	QIC4

sbr		r29,8		;burglar alarm r29,3(8) set


QIC4:

sts		$00E6,r29		;put data into present input status SRAM $00E6

ret		


;---------------------------

ClearRelayData:

push	r16

clr		r16
sts		$0080,r16
sts		$0081,r16		;clear send data

sts		$0073,r16
sts		$0075,r16
sts		$0077,r16
sts		$0087,r16		;clear received data

pop		r16

ret


;--------------------------------

ClearButtonPush:	;clears SRAM $00E7-$00EB of data put in by Button Push Routine

push	r16
clr		r16

sts		$00E7,r16
sts		$00E8,r16
sts		$00E9,r16
sts		$00EA,r16
sts		$00EB,r16

pop		r16

ret


;---------------------------

ReadOutByte:

;reads out byte in r16 from MSD to LSD

cbi		PortB,1
rcall	Delay2310
rcall	Delay2310
rcall	Delay2310
rcall	Delay2310
sbi		PortB,1		;long flash of LED

sbi		PortB,1		;off LED

rcall	Delay2310
rcall	Delay2310
rcall	Delay2310

push	r20

ldi		r20,8


ROB1:

sbrs	r16,7		;skip if bit is set
rjmp	ROB2

cbi		PortB,1		;on LED
rcall	Delay2310
sbi		PortB,1		;off LED
rcall	Delay2310
rcall	Delay2310
rjmp	ROB3


ROB2:

cbi		PortB,1
rcall	Delay495
sbi		PortB,1
rcall	Delay2310
rcall	Delay2310
rcall	Delay2310


ROB3:

lsl		r16
dec		r20
brne	ROB1

rcall	Delay2310
rcall	Delay2310
rcall	Delay2310

pop		r20

ret


;-------------------------------

SetUnitNumberRly1:		;sets new unit number for Relay #1 and records it simultaneously
						; into EEPROM as well

push	r30
push	r31

rcall	OneGreenFlash


SUN1a:

rcall	StallButtonPush
lds		r16,$00EA			;LSB of ButtonPush data into r16

sts		$0072,r16			;put unit number into Relay 1 SRAM
clr		r31					;zero r31

ldi		r30,$0020			;set up low byte for EEPROM write
rcall	WriteEEPROM

pop		r31
pop		r30

rcall	ShortBeep2

ret


;---------------------------------

SetUnitNumberRly2:

lds		r16,$006F			;load number of active relays
cpi		r16,2
brsh	SUN2b
rcall	LongRedFlash	;non-valid relay entry

ret


SUN2b:

push	r30
push	r31

rcall	OneGreenFlash	;indicate it is ready for one entry


SUN2a:

rcall	StallButtonPush
lds		r16,$00EA			;LSB of ButtonPush data into r16

sts		$0074,r16			;put unit number into Relay 2 SRAM
clr		r31					;zero r31

ldi		r30,$0021			;set up low byte for EEPROM write for relay 2
rcall	WriteEEPROM

pop		r31
pop		r30

rcall	ShortBeep2

ret


;------------------------------

SetUnitNumberRly3:

lds		r16,$006F		;load in number of active relasy
cpi		r16,3
brsh	SUN3b
rcall	LongRedFlash

ret					;return - non valid relay number entry


SUN3b:

push	r30
push	r31

rcall	OneGreenFlash	;indicate it is ready for one entry


SUN3a:

rcall	StallButtonPush
lds		r16,$00EA			;LSB of ButtonPush data into r16

sts		$0076,r16			;put unit number into Relay 3 SRAM
clr		r31					;zero r31

ldi		r30,$0022			;set up low byte for EEPROM write
rcall	WriteEEPROM

pop		r31
pop		r30

rcall	ShortBeep2

ret


;---------------------------

SetUnitNumberRly4:

lds		r16,$006F			;load number of active relays
cpi		r16,4
brsh	SUN4b					
rcall	LongRedFlash	;non-valid relay entry

ret


SUN4b:

push	r30
push	r31

rcall	OneGreenFlash	;indicate it is ready for one entry


SUN4a:

rcall	StallButtonPush
lds		r16,$00EA			;LSB of ButtonPush data into r16

sts		$0086,r16			;put unit number into Relay 4 SRAM
clr		r31					;zero r31

ldi		r30,$0023			;set up low byte for EEPROM write
rcall	WriteEEPROM

pop		r31
pop		r30

rcall	ShortBeep2

ret


;-------------------------

ShortBeep2:			;If this is called, two beeps are heard

sbi		PortB,3		;on sonalert
rcall	Delay50ms
cbi		PortB,3		;off sonalert
rcall	Delay50ms


;---

ShortBeep:

sbi		PortB,3		;on sonalert
rcall	Delay50ms
cbi		PortB,3		;off sonalert
rcall	Delay50ms

ret


;------------------------


LongBeep:

sbi		PortB,3		;on sonalert
rcall	Delay1sec
cbi		PortB,3		;off sonalert

ret


;---------------------------

ClearAllAlarms:			;Clears entire SRAM alarm memory and resets bookmark

push	r30
push	r31

clr		r31				;set up for use of register z low bytes only
ldi		r30,$90			;set to start of alarm time SRAM memory

ldi		r16,$90			;reset book mark $008F to start of memory
sts		$008F,r16

clr		r16				;zero r16


CAAlarms1:

cpi		r30,$D2			;check for end of memory
breq	CAAlarmsEnd

st		z+,r16			;put zero into alarm store area - post increment

rjmp	CAAlarms1


CAAlarmsEnd:

pop		r31
pop		r30

rcall	ShortBeep2

ret


;--------------------

SRAMtoEEPROM:		;puts SRAM alarm times into EEPROM


push	r28
push	r29
push	r30
push	r31			;Push y and z register pairs

clr		r29
clr		r31			;not used high bytes are cleared

ldi		r28,$8F		;start of bookmark & clock alarm section
ldi		r30,$2F		;start of EEPROM bookmark & clock alarm store area


STE1:

tst		r28			;check for end of clock alarm store area
cpi		r28,$D2		;compare end of memory +1
breq	STEend

ld		r16,y+		;load data from SRAM and post increment
rcall	WriteEEPROM	;write data into EEPROM (data in r16)
inc		r30

rjmp	STE1


STEend:

pop		r31
pop		r30
pop		r29
pop		r28

rcall	ShortBeep2

ret


;-------------------------	

EEPROMtoSRAM:

push	r28
push	r29
push	r30
push	r31			;Push y and z register pairs

clr		r29
clr		r31			;clear unused high bytes

ldi		r30,$2F		;start of EEPROM bookmark and clock time store (z reg)
ldi		R28,$8F		;start of bookmark and clock times in SRAM (y reg)


ETS1:

cpi		r30,$72		;check for end of EEPROM memory area +1
breq	ETSend
rcall	FetchEEPROM	;Fetch EEPROM using z register and r16 containing returned data
inc		r30
st		y+,r16		;store data in r16 into SRAM with post increment
rjmp	ETS1


ETSend:

pop		r31
pop		r30
pop		r29
pop		r28

rcall	ShortBeep2

ret


;--------------------------

LivedInToggle:		;toggle lived in flag, r17,6(64) - 0 = off

sbrs	r17,6		;skip if r17,6(64) is set (lived-in active)
rjmp	LInt1

rcall	OffAllDuskDawnFunctions	;clear flags and counters

rcall	LongBeep

rjmp	LIntEnd


LInt1:

rcall	OffAllDuskDawnFunctions	;clear flags and counters

sbr		r17,64		;set r17,6(64)
rcall	ShortBeep


LIntEnd:

ret


;-----------------------

NiteLiteToggle:		;Toggle night light flag on/off - 0=off, 1=on

sbrs	r17,7		;skip if r17,7(128) is set 1= active
rjmp	NLite1

rcall	OffAllDuskDawnFunctions	;clear flags and counters

rcall	LongBeep
rjmp	NLiteEnd


NLite1:

rcall	OffAllDuskDawnFunctions	;clear flags and counters

sbr		r17,128		;set r17,7(128) to active - night light on
rcall	ShortBeep


NLiteEnd:

ret


;-----------------------

BurglarAlarmToggle:		;toggles BA flag on/off r18,7(128)

sbrs	r18,7			;skip if r18,7(128) is set (on)
rjmp	BAM1

rcall	OffAllDuskDawnFunctions	;clear flags and counters

rcall	LongBeep
rjmp	BAMend


BAM1:

rcall	OffAllDuskDawnFunctions	;clear flags and counters

sbr		r18,128			;set r18,7(128) to ON
rcall	ShortBeep


BAMend:

ret


;----------------------

LivedInTwoToggle:		;toggles Lived In 2 on and off

sbrs	r18,3		;skip if r18,3 is set (lived in 2 enabled)
rjmp	LItwo1

rcall	OffAllDuskDawnFunctions	;clear flags and counters

rcall	LongBeep
rjmp	LItwoEnd


LItwo1:

rcall	OffAllDuskDawnFunctions	;clear flags and counters

sbr		r18,8		;set r18,3(8) to enable lived in 2
rcall	ShortBeep


LItwoEnd:

ret


;-------------------------

LivedInThreeToggle:		;toggles Lived In 3 on and off

sbrs	r26,5		;skip if r26,5(32) is set (lived in 3 enabled)
rjmp	LIthree1

rcall	OffAllDuskDawnFunctions	;clear flags and counters
rcall	LongBeep
rjmp	LItwoEnd


LIthree1:

rcall	OffAllDuskDawnFunctions	;clear flags and counters

sbr		r26,32		;set r26,5(32) to enable lived in 3
rcall	ShortBeep


LIthreeEnd:

ret


;-------------------------

TestDuskDawn:		;allows one to check potentiometer setting for detection
					;  green LED on when day, red LED on when night

rcall	ShortBeep2


TestDD1:

sbi		PortD,7		;on voltage to CDS cell

sbis	PIND,2		;skip if PIND,2 is set (touch plate not touched
rjmp	TDD2

sbis	PIND,1		;skip in PIND,1 is set (it is night)
rjmp	TDD1

sbi		PortB,2		;off green LED, portB,2(4)
cbi		PortB,1		;on red LED, portB,1(2)

rjmp	TestDD1


TDD1:

sbi		PortB,1		;off red LED, PortB,1(2)
cbi		PortB,2		;on green LED, PortB,2(4)

rjmp	TestDD1


TDD2:

sbi		PortB,1
sbi		PortB,2		;off red and green LEDs

cbi		PortD,7		;off voltage to CDS cell

rcall	ButtonStall	;wait until button is released before returning

rcall	ShortBeep2

ret


;-----------------------
					
ClearEEPROMalarms:  ;clears all data in EEPROM alarm area

push	r30
push	r31			;Push y and z register pairs

clr		r31			;not used high bytes are cleared

ldi		r30,$2F		;start of EEPROM bookmark & clock alarm store area

rcall	WriteEEPROM
inc		r30			;ready for clearing of all EEPROM data locations
					;  starting at $0030 EEPROM address

clr		r16			;zero for store of data


ClrEEPROM1:

tst		r30			;check for end of clock alarm store area

cpi		r30,$72		;compare end of memory +1
breq	ClrEEPROMend

rcall	WriteEEPROM	;write data into EEPROM (data in r16)
inc		r30

rjmp	ClrEEPROM1


ClrEEPROMend:

pop		r31
pop		r30

rcall	ShortBeep2

ret


;-------------------------	


ProximityToggle:	;toggle proximity alarm flag on or off - 0 = off

sbrs	r29,6		;skip if prox flag, r29,6(64) is set (active)
rjmp	ProxT1

rcall	OffAllDuskDawnFunctions	;clear flags and counters

rcall	LongBeep
rjmp	ProxTend


ProxT1:

rcall	OffAllDuskDawnFunctions	;clear flags and counters

sbr		r29,64		;set r29,6(64) - prox detection active
cbr		r18,128		;clear r18,7(128) - burglar alarm flag (only can be active 
					;  prox or burg alarm
rcall	ShortBeep


ProxTend:

ret


;------------------------

ZeroAllFlags:		;sets all flags to default zero setting

clr		r16

mov		r5,r16
mov		r17,r16
mov		r18,r16
mov		r19,r16
mov		r26,r16

rcall	ShortBeep

ret


;----------------------------

OneSecondFlagStall:	;Used when Master unit flag is active (r19,3(8))=1
					;  to sync up again
					;Waits for one second flag to be set

cbr		r19,128		;clear r19,7(128) one second flag


OSFS2:

sbrs	r19,7		;skip if r19,7(128) is set (one second flag)
rjmp	OSFS2

ret


;-----------------------

StoreLivedInTimeOff:

push	r20
push	r28
push	r29
push	r30
push	r31

clr		r28
clr		r29
clr		r30
clr		r31

ldi		r16,2
rcall	FlashGLED		;four green flashes indicating two entries are needed

rcall	StallButtonPush	;24 hour format - 1 = ZERO,  2 = 1 hr.,  3 = 2 hrs., etc.

lds		r20,$00EA		;get total count from $00EA (It will be greater than zero)
dec		r20				;one less to made 1 = 0 and one less on all other entries


SLOff3:

tst		r20
breq	SLOff4
dec		r20
adiw	r28,60				;add 60 to r28,r29
rjmp	SLOff3


SLOff4:

rcall	OneGreenFlash		;to show another entry is needed -- minutes
rcall	StallButtonPush
lds		r20,$00EA			;get total minutes count
dec		r20					; to make 1 = 0,  2 = 1,   3 = 2,  etc.


SLOff7:

add		r28,r20			;add r20 to r28
brcs	SLOff7a
rjmp	SLOff8


SLOff7a:

inc		r29				;for carry of one from add


SLOff8:

sts		$0088,r28
sts		$0089,r29			;store away new time off for lived in - in SRAM


clr		r31				;zero MSbit

mov		r16,r28
ldi		r30,$25			;EEPROM storage location for LSB
rcall	WriteEEPROM		;store away LSB in r28

mov		r16,r29
ldi		r30,$26			;EEPROM storage location for MSB
rcall	WriteEEPROM		;store away MSB in r29

pop		r31
pop		r30
pop		r29
pop		r28
pop		r20

rcall	ShortBeep2

ret


;---------------------------------

FetchLivedInOffTime:	;Fetches time from EEPROM for lived in times

push	r30
push	r31		;push y & z register pairs

ldi		r30,$25
ldi		r31,0

rcall	FetchEEPROM

sts		$0088,r16		;fetch LSB of Lived In Off time and put in SRAM

inc		r30
rcall	FetchEEPROM

sts		$0089,r16		;fetch MSB of Lived In Off time and put in SRAM

pop		r31
pop		r30

ret


;----------------------------------

AutoOffRelay1Control:	;Only relay #1 has this capability.
						;Entry of one DISABLES this delay.
						;  All other entries valid.  Two entry gives two minutes on, etc.
						;  which is minimum time on permitted.
						
push	r28
push	r29
push	r30
push	r31
		
rcall	ButtonPushX60

sbiw	r28,60			;subtract 60 seconds from r28,29 y register pair
						;  to determine if one was entered			
breq	AORC1			;zero entry = disable

adiw	r28,60			;add back in 60 seconds - minimum on time is two minutes


AORC1:

ldi		r30,$002D		;EEPROM store location for LSD - Auto Off of Relay1 Location
clr		r31

mov		r16,r28
rcall	WriteEEPROM
inc		r30
mov		r16,r29
rcall	WriteEEPROM		;Write seconds on data in R28,29 to EEPROM

sts		$0062,r28
sts		$0063,r29		;store in permanent store, 10X number

sts		$007A,r28
sts		$007B,r29		;store in countdown timer, 10X number

rcall	ShortBeep2		;indicate correct entry


AORCend:
pop		r31
pop		r30
pop		r29
pop	`	r28

ret


;------------------------------

SetBATimeOn:		;Sets minutes on for relay 1 when Burglar Alarm is
					;  triggered. 
					;R28,R29 contains multiplied seconds data from ButonPushX60
			
push	r28
push	r29
push	r30
push	r31
		
rcall	ButtonPushX60

ldi		r30,$0029		;EEPROM store location for LSD - Burglar Alarm Time On
clr		r31

mov		r16,r28
rcall	WriteEEPROM
inc		r30
mov		r16,r29
rcall	WriteEEPROM		;Write seconds on data in R28,29 to EEPROM

rcall	ShortBeep2		;indicate correct entry

pop		r31
pop		r30
pop		r29
pop	`	r28

ret


;--------------------------------

SetProxTimeOn:			;Sets minutes on for relay 1 when Proximity Detector
						;  is triggered.  R28, R29 contain multiplied seconds data
						;  from ButtonPushX60 routine.

push	r28
push	r29
push	r30
push	r31
		
rcall	ButtonPushX60

ldi		r30,$002B		;EEPROM store location for LSD - Prox Detector Time On
clr		r31

mov		r16,r28
rcall	WriteEEPROM
inc		r30
mov		r16,r29
rcall	WriteEEPROM		;Write seconds on data in R28,29 to EEPROM

rcall	ShortBeep2		;indicate correct entry

pop		r31
pop		r30
pop		r29
pop		r28

ret


;------------------------

LoadDefaultUnitNumbers:		;Loads default unit numbers at bootup
							;Master = Unit 12
							;Slave = Units 31, 32, 33, 34
push	r30
push	r31

sbrs	r19,3				;skip if r19,3(8) is set (master)
rjmp	LDUN1

ldi		r16,12
sts		$0072,r16			;put unit number is SRAM
ldi		r30,$20				;EEPROM location for relay 1
ldi		r31,0
rjmp	LDUNend


LDUN1:

rcall	writeEEPROM			;store master unit number away

ldi		r16,31
sts		$0072,r16			;put unit number is SRAM
ldi		r30,$20				;EEPROM location for Relay 1
rjmp	LDUNend

rcall	writeEEPROM			;store away slave unit number for relay 1

ldi		r16,32
sts		$0074,r16			;put unit number is SRAM
ldi		r30,$21				;EEPROM location for Relay 2

rcall	writeEEPROM			;store away slave unit number for relay 2

ldi		r16,33	
sts		$0076,r16			;put unit number is SRAM
ldi		r30,$22				;EEPROM address for Relay 3

rcall	writeEEPROM			;store away slave unit number for relay 3

ldi		r16,34
sts		$0086,r16			;put unit number is SRAM
ldi		r30,$23				;EEPROM address for Relay 4				

rcall	writeEEPROM			;store away slave unit number for relay 4


LDUNend:

pop		r31
pop		r30

rcall	ShortBeep2

ret


;-------------------------------

WriteDefaultLivedInOffTime:		;writes 11;00 p.m. lived in off time into EEPROM and SRAM

push	r30
push	r31

ldi		r30,$0025
ldi		r31,0			;put in address for Lived In Off Time in EEPROM

ldi		r16,100
rcall	WriteEEPROM
sts		$0088,r16

inc		r30
ldi		r16,5
rcall	WriteEEPROM
sts		$0089,r16		;store complete. EEPROM and SRAM now have 11:00 p.m. off time

pop		r31
pop		r30

ret


;-------------------------

ReadInUnitNumbers:		;read in unit numbers from EEPROM (at bootup)

push	r30
push	r31

ldi		r30,$20			;start of four byte store area in EEPROM
clr		r31				;zero high byte

rcall	FetchEEPROM
sts		$0072,r16
inc		r30
rcall	FetchEEPROM
sts		$0074,r16
inc		r30
rcall	FetchEEPROM
sts		$0076,r16
inc		r30
rcall	FetchEEPROM
sts		$0086,r16			;All relays now have unit numbers restored into SRAM

pop		r31
pop		r30

ret


;-----------------------------------------

ReadInLivedInOffTime:

push	r30
push	r31

ldi		r30,$25
clr		r31

rcall	FetchEEPROM
sts		$0088,r16		;low byte
inc		r30
rcall	FetchEEPROM
sts		$0089,r16		;high byte of live in time off (11:00 p.m. default)

pop		r31
pop		r30

ret


;-------------------------------------------

ReadInClockCorrectionInterval:	;read in seconds and interval for clock correction
								;one second added at this interval to speed up clock

								;R28=$10, r29=$11,  r30=$12,  r31=$13
								;  Mode (no(0), minus on second(1), plus one second(2))=$0114
								;SRAM: r28=$0110,0114,  r29=$0111,0115,  r30=$0112,0116
								;  r31=$0113,0117,   Mode=$0118

push	r30
push	r31

ldi		r30,$10
clr		r31

rcall	FetchEEPROM
sts		$0110,r16				;low interval byte to permanent store
sts		$0114,r16				;low interval byte to counter

inc		r30
rcall	FetchEEPROM


sts		$0111,r16				;high interval byte to permanent store
sts		$0115,r16

inc		r30
rcall	FetchEEPROM

sts		$0112,r16
sts		$0116,r16

inc		r30
rcall	FetchEEPROM		;Get final high byte of high byte pair, r30,31

sts		$0113,r16
sts		$0117,r16

inc		r30
rcall	FetchEEPROM		;Get add/subtract one second (bit 128 = 0 for subtract)

sts		$0118,r16		;speed up 3, slow down 2,  0 = no change

pop		r31
pop		r30

ret


;------------------------

ClearUnusedRelayEEPROM:

push	r17
push	r30
push	r31

lds		r16,$006F		;load in number of relays this unit uses
ldi		r17,4
sub		r17,r16		;subtract r16 from r17 and put result in r17
ldi		r30,$23		;load top of EEPROM memory location into low Z byte
clr		r31			;zero high byte of z register


CURE1:

cpi		r17,0
brne	CURE2

pop		r31
pop		r30
pop		r17

ret


CURE2:

clr		r16				;zero r16 for zero entry in non functional relay positions
rcall	WriteEEPROM		;write zero into location pointed by R30,31
dec		r30
dec		r17

rjmp	CURE1


;------------------------

OffAllDuskDawnFunctions:	;turns off special functions - Lived In, Lived In 2, 
							; Lived In 3, Burglar Alarm
							; Proximity Alarm, Dusk/Dawn, Nite Light

cbr		r18,152			;clear Lived In 2 enable and "in process" flags, Burglar alarm flag
cbr		r26,96			;clear Lived In 3 enable and "in process" flags
cbr		r17,64			;clear lived in flag
cbr		r17,128			;clear night light flag
cbr		r29,64			;clear proximity detector flag


clr		r16
sts		$007C,r16			;Burglar Alarm counter
sts		$007D,r16
sts		$007E,r16			;Prox Detector counter
sts		$007F,r16
sts		$0082,r16			;Lived in Random count down number
sts		$008A,r16			;Burglar Alarm count down
sts		$008B,r16
sts		$00DE,r16			;Dusk/Dawn 10 min counter

sts		$00EC,r16			;Prox Det Ignore counter
sts		$00ED,r16			;Prox Det On counter
sts		$00EE,r16
sts		$00EF,r16			;Relay 1 auto off counter

sts		$00F0,r16
sts		$00F1,r16			;zero lived in 2 ON counter
sts		$00F2,r16
sts		$00F3,r16			;zero lived in 2 OFF counter
sts		$00F4,r16			;zero Final Lived In 2 Off Counter
sts		$00F6,r16			;zero lived in 3 On counter
sts		$00F7,r16			
sts		$00F8,r16			;zero livedin 3 Off counter
sts		$00F9,r16
sts		$00FA,r16			;zero Final lived in 3 Off Counter

push	r30
push	r31

ldi		r30,$25
clr		r31					;EEPROM store location for off time data

rcall	FetchEEPROM
sts		$0088,r16

inc		r30

rcall	FetchEEPROM
sts		$0089,r16			;Off time for lived in functions now in SRAM $0088,89

ldi		r30,$0015

rcall	FetchEEPROM
sts		$00FE,r16

inc		r30

rcall	FetchEEPROM
sts		$00FF,r16				;grab Lived In 2 On/Off delays from EEPROM

pop		r31
pop		r30

ret


;-------------------------------------


SetLivedIn2OnOffProportions:	;Sets proportion of light on to light off for Lived In 2 operation
								;  Entry range is from 1 to 15 -- 1 being short on time, 15 being long on time
								;  1 = Min ON 10 minutes (maximum off = 74 min)
								;  15 = Min ON 74 minutes (maximum off = 10 min)
push	r17
push	r30
push	r31	
							;  

rcall	OneGreenFlash

rcall	StallButtonPush
lds		r16,$00EA		;get LSB data from ButtonPush

inc		r16				;boost value by one from that entered
cpi		r16,16
brsh	SLIOPEnd1		;exit: ERROR input

sts		$00FF,r16		;store in ON delay time location

ldi		r30,$0016
ldi		r31,0

rcall	WriteEEPROM		;write ON delay to EEPROM $0016

ldi		r17,17
sub		r17,r16			;subtract r16 from r17
mov		r16,r17			;move r17 value into r16
sts		$00FE,r16		;put into SRAM, ON delay
dec		r30				;point to OFF delay time location

rcall	WriteEEPROM		;permanently store in EEPROM

rcall	ShortBeep2

rjmp	SLIOPEnd


SLIOPEnd1:

rcall	LongBeep		;Long beep to indicate error entry


SLIOPEnd:

pop		r31
pop		r30
pop		r17

ret


;-------------------------------------------

GrabAutoOffTimeInEEPROM:	;Automatic off of relay
							;If zero, this function is disabled
							;  any other number gives seconds off for relay 1
push	r30
push	r31


ldi		r30,$002D		;EEPROM store location for LSD - Auto Off of Relay1 Location
clr		r31

rcall	FetchEEPROM
sts		$0062,r16
sts		$007A,r16
inc		r30
rcall	FetchEEPROM		;Write seconds on data in R28,29 to EEPROM
sts		$0063,r16
sts		$007B,r16

pop		r31
pop		r30

ret


;---------------------------------------------

SomeDefaultSettings:			;Resets flags, clears auto off delay for relay 1,  

clr		r5			;clear flags in register r5
clr		r6			;clear day-night data register
clr		r7			;clear 4am pause counter
clr		r8			;clear 4am pulse counter
clr		r9			;clear Dusk/Dawn pulse counter
clr		r11			;clear Proximity Detector Counter - pause
clr		r12			;;clear Burglar Alarm Counter - pause
clr		r14			;dusk/dawn pulse count - one count = daylight, two = dusk
clr		r17			;clear flags in register r17
clr		r18			;clear flags in register r18
clr		r19
clr		r20			;quick input check data	
clr		r21			;clear flags register
clr		r26			;master inputs check - pulse received bits
clr		r29			;QuickInputsCheck store	
clr		r30
clr		r31			;clear automatic off for relay one counter (r30,r31)	

clr		r16
sts		$0080,r16		;start with no unit number for touch plate command
sts		$0081,r16		;no data in touch plate command.
sts		$0083,r16		;zero data for zero command
sts		$008A,r16		;zero burglar alarm countdown count
sts		$008B,r16		;zero proximity detector countdown count
sts		$00DE,r16		;dusk-dawn 10 minute counter
sts		$006D,r16		;beep/relay number for slot
sts		$006E,r16		;beep/relay number for loop end processing
sts		$0078,r16		;dawn counter
sts		$0079,r16		;dusk counter
sts		$007A,r16		;Auto Relay 1 off LSB
sts		$007B,r16		;Auto Relay 1 off MSB
sts		$007C,r16		;B.A. Relay 1 off location             
sts		$007D,r16		;B.A. Relay 1 off MSB
sts		$007E,r16		;Prox Relay 1 off LSB
sts		$007F,r16		;Prox Relay 1 off MSB
sts		$00FB,r16		;zero 8 counter for 4am detection

ldi		r16,$90			;book mark start of alarm memory
sts		$008F,r16		;store in book mark memory location

rcall	ClearUnusedRelayEEPROM	;zeros EEPROM locations for inactive relays

ret


;---------------

DoDefaultReset:			;Reset all alarms and give unit 100 as #1 relay assignment

rcall	SomeDefaultSettings

rcall	ClearAllAlarms

push	r30
push	r31

clr		r31		;clear r31, max address byte

ldi		r30,$20	;Unit location for relay #1

ldi		r16,100	;Unit number assignment as #100

rcall	WriteEEPROM

pop		r31
pop		r30

ret


;-------------

QuickSetDisplayAlarm:	;Sets 15 minutes segments from midnight into alarm location
					;  for Display Clock Alarm.  Note - regular loacal alarms set
					;  during slave operation still effective.

push	r30
push	r31

rcall	OneGreenFlash	;One entry needed

clr		r30
clr		r31				;clear to zero, z register pair, r30,31

rcall	StallButtonPush
lds		r16,$0100		;get LSB data from ButtonPush (One Less) $0100 low byte



QSD1:

tst		r16
breq	QSD2

adiw	r30,15			;add fifteen minute segment to z register, r30,31

dec		r16

rjmp	QSD1


QSD2:

sts		$0119,r30
sts		$011A,r31		;store in SRAM display alarm location

pop		r31
pop		r30

rcall	ShortBeep2

ret


;-----------------------------------------


;Routines Below This Line are for TEST purposes or OLD code only........


;---------------------------------------

ToggleB5:			;toggle LED B5

sbis	PortB,5
rjmp 	TB5a

cbi		PortB,5
ret


TB5a:

sbi		PortB,5
ret

;---------------------------------

ToggleB4:

sbis	PortB,4
rjmp	TB4a

cbi		PortB,4
ret


TB4a:

sbi		PortB,4
ret


;-------------------------- 

ToggleB3:

sbis	PortB,3
rjmp	ToggleB3a

cbi		PortB,3
ret


ToggleB3a:

sbi		PortB,3
ret


;---------------------------


